# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import osv, fields, orm
from lxml import etree
from datetime import datetime
from tools.translate import _
import wizard
import pooler


# Utility function for convert a database format date string to python date object
def str_to_date(dt):
    return date(int(dt[0:4]),int(dt[5:7]),int(dt[8:10]))

# Utility function for convert a database format datetime string to python datetime object
def str_to_datetime(dt):
    return datetime(int(dt[0:4]),int(dt[5:7]),int(dt[8:10]),int(dt[11:13]),int(dt[14:16]))

ABSENCE_CODE_STATE='absent'

class res_partner_contact(osv.osv):
    _name = "res.partner.contact"
    _inherit = "res.partner.contact"

    _columns = {
        'foto': fields.binary('Foto', filters='*.png,*.jpg' ),
    }


res_partner_contact()


class school_attendance_aux_fields_view(osv.osv):
    _name = 'school_attendance.aux.fields_view'

    def _check_correct(self, cr, uid, ids, field_name, arg, context=None):
        def _state_in_states(state,states):
            # @type states str
            sign=False
            for stati in states.split(','):
                if len(stati)<1: continue
                if stati[0]=='-' and stati[1:]==state:
                    sign='-'
                elif stati[1:]==state or stati==state:
                    sign='+'
            return sign
        
        for item in self.browse(cr, uid, ids):
            ret={}
            ret[item.id]=_state_in_states(item.check_state,item.anno_add.states)=='+'\
                and _state_in_states(item.check_state,item.anno_subs.states)=='-'
        return ret
        
    _columns = {
        'name' : fields.char('Name',size=100,translate=True,),
        'anno_add' : fields.many2one('contact.type_annotation','Annotation more', help="Annotation to add on student"),
        'anno_subs' : fields.many2one('contact.type_annotation','Annotation less', help="Annotation to subtract on student"),
        'check_state' : fields.char('check_state',size=30,),
        'sequence' : fields.integer('Sequence',select=1,),
        'correct' : fields.function(_check_correct, type='boolean', method=True, string='Correct', store={
            'school_attendance.aux.fields_view': (lambda self, cr, uid, ids, context: ids,['anno_add','anno_subs','check_state',],10),
            'contact.type_annotation': (lambda self, cr, uid, ids, context: self.pool.get('school_attendance.aux.fields_view').search(cr, uid, ['|',('anno_add','in',ids),('anno_subs','in',ids)]) ,['data_states'],10),
            'contact.type_anno.state': (lambda self, cr, uid, ids, context: self.pool.get('school_attendance.aux.fields_view').search(cr, uid, ['|',('anno_add.data_states','in',ids),('anno_subs.data_states','in',ids)]) ,['op','state_id'],10)
        },)
    }

    _order='sequence'
school_attendance_aux_fields_view()


PREFIXE='state'

class update_seance_attendance(osv.osv_memory):

    user_manager_id=1

    def __init__(self, pool, cr):
        osv.osv_memory.__init__(self, pool, cr)
        self.more_datas = {}

    def name_get(self, cr, uid, ids, context=None):
        ret=[]
        items=self.browse(cr, uid, ids)
        name_by_seance_id={}
        for (seance_id, name) in self.pool.get('school.seance').name_get(cr, uid, [x.seance_id.id for x in items]):
            name_by_seance_id[seance_id]=name
        for item in items:
            ret.append( (item.id, name_by_seance_id[item.seance_id.id]) )
        return ret
            
    def _num_students(self, cr, uid, ids, field_name, arg, context=None):
        ret={}
        for item in self.browse(cr, uid, ids):
            seance=item.seance_id
            ret[item.id]=len(seance.contact_ids)
        return ret
        
    _name = 'wizard.school_attendance.update_seance_attendance'
    _columns = {
        'seance_id' : fields.many2one('school.seance','Seance',readonly=True,),
        'date' : fields.related('seance_id','date',type='datetime',string="Date",readonly=True,),
        'room' : fields.related('seance_id','room_id',type='many2one',relation='school.room',string="Room",readonly=True,),
        'num_students' : fields.function(_num_students,type='integer',method=True,string='Num. students'),
        'last_attendance_update' : fields.related('seance_id','last_attendance_update',type='datetime',string="Last Update",readonly=True,),
        'user_attendance_update' : fields.related('seance_id','user_attendance_update',type='many2one',relation='res.users',string="Last Update User",readonly=True,),
        'state' : fields.selection( [('new','New'),('executed','Executed')] , ),
        'model_' : fields.boolean('Model',),
    }

    _defaults = {'state': lambda *a: 'new',}

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
#        print datetime.today(),"update_seance_attendance: search: args, offset=0, limit=None, order=None, context=None, count : ", args, offset, limit, order, context, count
        if count: return 1
        return []

    def _fields_more(self, cr, uid, context=None):
#        print datetime.today(),"update_seance_attendance: fields_more: context : ", context
        obj=self.pool.get('school_attendance.aux.fields_view')
        return obj.read(
                cr, self.user_manager_id,
                obj.search(cr, self.user_manager_id, [('correct','=',True)]),
                ['check_state','name','anno_add','anno_subs'], context=context)

    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
#        print datetime.today(),"update_seance_attendance: fields_view_get: view_id=None, view_type='form', context=None, toolbar=False, submenu : ", view_id, view_type, context, toolbar, submenu

        """
        paràmetres en el contexte:
        - 'amb_foto' : True or False si volem amb foto o sense . Només en tipus_formulari=0.
        - 'with_comments' : True or False si volem amb comentari o sense . Només en tipus_formulari=0.
        - 'tipus_formulari' : 1 compacte, extens un altre valor
        - 'num_rows_before_header' : s'explica sol. Espera un número. Només pel tipus compacte.
        - 'no_headers_with_initials' : True or False . Llengenda i abrevitures per cada casella. Només pel tipus compacte.
        """
        res = super(update_seance_attendance, self).fields_view_get(cr, uid, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar, )
        act_id=self.pool.get('ir.actions.act_window').search(cr, uid, [('res_model','=','contact.annotation'),('usage','=','wizard_attendance')] )[0]
        act_id2=self.pool.get('ir.actions.act_window').search(cr, uid, [('res_model','=','contact.annotation'),('usage','=','new_annotation_from_wizard')] )[0]
        if view_type!='form': return res
        fields_more=self._fields_more(cr, uid, context=context)
        cols=len(fields_more)
        view_obj = etree.XML(res['arch'])
        for child in view_obj:
            if context.get('tipus_formulari',1)==1:
                if context.get('no_headers_with_initials',False): cols=cols*2
                if child.tag=='group' and child.get('col')=='99':
                    del child[:]
                    obj=self.pool.get('school.seance').browse(cr, uid, context.get('seance_id',context.get('active_id',False)))
                    child.set('col',"3")
                    group1=etree.SubElement(child,'group',col="1",width="50%",)
                    etree.SubElement(group1,'label',string="Name",align="0.0",)
                    if context.get('no_headers_with_initials',False):
                        group2=etree.SubElement(child,'group',col=str(cols),width="30%",)
                        for field in fields_more:
                            etree.SubElement(group2,'label',string=field['name'],align="0.0",)
                            etree.SubElement(group2,'label',string='('+field['name'][0]+')',align="0.0",)
                    else:
                        group2=etree.SubElement(child,'group',col=str(cols),width="30%",)
                        for field in fields_more:
                            etree.SubElement(group2,'label',string=field['name'],align="0.0",)

                    group3=etree.SubElement(child,'group',col="1",width="20%",)
                    etree.SubElement(group3,'label',string="Annotations",align="0.0",)
                    etree.SubElement(child,'separator',colspan="3",)
                    contacts=sorted(obj.contact_ids, key=lambda x: x.name+x.first_name)
                    counter_contacts=0
                    for contact in contacts:
                        group1=etree.SubElement(child,'group',col="1",width="50%",)
                        name=''
                        if contact.name:
                            name+=contact.name
                        if contact.first_name:
                            name+=', '+contact.first_name
                        etree.SubElement(group1,'label',string=name,align="0.0",)
                        group2=etree.SubElement(child,'group',col=str(cols),width="30%",)
                        for field in fields_more:
                            # TODO: on_change='_change_field()',
                            if context.get('no_headers_with_initials',False):
                                etree.SubElement(group2,'label',string=field['name'][0],align="0.0",)
                            etree.SubElement(group2,'field',nolabel="1",type='boolean', name=PREFIXE+'_'+field['check_state']+'_'+str(contact.id))
                        group3=etree.SubElement(child,'group',col="2",width="20%",)
                        invisible="1"
                        for anno in obj.annotation_ids:
                            if anno.contact_id.id==contact.id: invisible="0"
                        etree.SubElement(group3,'button',string=_("Annotations"),align="0.0",type="action",name=str(act_id),context="{'domain_contact_id': %s,'domain_valid_from': '%s','domain_valid_to': '%s'}" % (contact.id,obj.date,obj.date_to),target="new",invisible=invisible,)
                        etree.SubElement(group3,'button',string=_("New Annotation"),align="0.0",type="action",name=str(act_id2),context="{'default_contact_id': %s,'default_valid_from': '%s','default_valid_to': '%s'}" % (contact.id,obj.date,obj.date_to),target="new",)
                        counter_contacts+=1
                        if counter_contacts>=context.get('num_rows_before_header',10) and not context.get('no_headers_with_initials',False):
                            counter_contacts=0
                            etree.SubElement(child,'separator',colspan="3",)
                            group1=etree.SubElement(child,'group',col="1",width="50%",)
                            etree.SubElement(group1,'label',string="Name",align="0.0",)
                            group2=etree.SubElement(child,'group',col=str(cols),width="30%",)
                            for field in fields_more:
                                etree.SubElement(group2,'label',string=field['name'],align="0.0",)
                            group3=etree.SubElement(child,'group',col="1",width="20%",)
                            etree.SubElement(group3,'label',string="Annotations",align="0.0",)
                        etree.SubElement(child,'separator',colspan="3",)


            else:

                if child.tag=='group' and child.get('col')=='99':
                    del child[:]
                    child.set('col',"2")
                    obj=self.pool.get('school.seance').browse(cr, uid, context.get('seance_id',context.get('active_id',False)))
                    contacts=sorted(obj.contact_ids, key=lambda x: x.name+x.first_name)
                    for contact in contacts:
                        if context.get('amb_foto',False):
                            group1=etree.SubElement(child,'group',col="2",width="100",)
                            etree.SubElement(group1,'field',name="foto_%s" % contact.id,nolabel="1",readonly="1",widget="image",width="67",height="100")
                        else:
                            group1=etree.SubElement(child,'group',col="1",width="100",)
                        group0=etree.SubElement(child,'group',col="1",)
                        group3=etree.SubElement(group1,'group',col="3",)
                        name=''
                        if contact.name:
                            name+=contact.name
                        if contact.first_name:
                            name+=', '+contact.first_name
                        etree.SubElement(group3,'label',string=name,align="0.0",width="200",)
                        invisible="1"
                        for anno in obj.annotation_ids:
                            if anno.contact_id.id==contact.id: invisible="0"
                        etree.SubElement(group3,'button',string=_("Annotations"),width="167",type="action",name=str(act_id),context="{'domain_contact_id': %s,'domain_valid_from': '%s','domain_valid_to': '%s'}" % (contact.id,obj.date,obj.date_to),target="new",invisible=invisible,)
                        etree.SubElement(group3,'button',string=_("New Annotation"),width="167",type="action",name=str(act_id2),context="{'default_contact_id': %s,'default_valid_from': '%s','default_valid_to': '%s'}" % (contact.id,obj.date,obj.date_to),target="new",)
                        if context.get('with_comments',True):
                            etree.SubElement(group3,'label',colspan="3",string='',)
                            etree.SubElement(group3,'label',colspan="3",string='',)
                            etree.SubElement(group3,'label',colspan="3",string=_('New annotation comments'),)
                        group5=etree.SubElement(group0,'group',col=str(cols),)
                        for field in fields_more:
                            etree.SubElement(group5,'label',string=field['name'],align="0.0",)
                        group2=etree.SubElement(group0,'group',col=str(cols),row="2")
                        for field in fields_more:
                            # TODO: on_change='_change_field()',
                            etree.SubElement(group2,'field',nolabel="1",type='boolean', name=PREFIXE+'_'+field['check_state']+'_'+str(contact.id))
                        if context.get('with_comments',True):
                            etree.SubElement(group2,'field',nolabel="1", name='comment_%s' % contact.id,colspan=str(cols))
                        etree.SubElement(child,'separator',colspan="2",)
        res['arch'] = etree.tostring(view_obj)
        return res


    def fields_get(self, cr, uid, fields=None, context=None):
        if not context: context={}
        res = super(update_seance_attendance, self).fields_get(cr, self.user_manager_id, fields, context=context)
        seance_id=context.get('seance_id',context.get('active_id',None))
        if not seance_id: return res
        seance=self.pool.get('school.seance').browse(cr, self.user_manager_id, seance_id)
        fields_more=self._fields_more(cr, uid)
        contact_ids=set([x.id for x in seance.contact_ids])
        contact_ids.update([x['student'][0] for x in self.pool.get('school.presence').read(cr, uid, self.pool.get('school.presence').search(cr, uid, [('seance_id','=',seance.id)]), ['student'])])
        for contact_id in contact_ids:
            for field in fields_more:
                res[PREFIXE+'_'+field['check_state']+'_'+str(contact_id)]={'type': 'boolean', 'string': '', 'domain':[], 'selectable': False, }
            if context.get('amb_foto',False):
                res['foto_%s' % contact_id]={'type': 'binary', 'string': '',}
            if context.get('with_comments',True):
                res['comment_%s' % contact_id]={'type': 'char', 'string': '','size': 200,}
        return res

    def read(self, cr, uid, ids, fields_to_read=None, context=None, load=None):
#        print datetime.today(),"update_seance_attendance: read: ids, fields_to_read=None, context=None, load : ", ids, fields_to_read, context, load
        if not context:
            context = {}
        if not fields_to_read:
            fields_to_read = self._columns.keys()
        fields2=filter(lambda x: x in self._columns, fields_to_read)
        if fields2:
            ret=super(update_seance_attendance, self).read(cr, uid, ids, fields2 )
        else:
            ret=[]
            for id in ids: ret.append({'id': id})

        ret2=[]
        for item in ret:
            if 'model_' in fields_to_read:
                item.update(self.more_datas[item['id']])
            foto_contact_ids=set()
            for field in fields_to_read:
                parts=field.split('_')
                if len(parts)>1 and parts[0]=='foto': foto_contact_ids.add(int(parts[-1]))
            for item_foto in self.pool.get('res.partner.contact').read(cr, uid, list(foto_contact_ids), ['foto']):
                data_foto=item_foto['foto']
                if data_foto and context.get('bin_size', False): data_foto = len(data_foto)
                elif context.get('bin_size', False): data_foto = 0
                item['foto_%s' % item_foto['id']]=data_foto
            ret2.append(item)
        return ret2

    def write(self, cr, uid, ids, vals, context=None):
#        print datetime.today(),"update_seance_attendance: write: ids, vals, context : ", ids, vals, context
        # @type vals dict
        if isinstance(ids, (int, long)):
            ids = [ids]
        ret=super(update_seance_attendance, self).write(cr, uid, ids, dict(filter(lambda (key,value): key in self._columns, vals.items() ) ) )
        for id in ids:
            for (key,value) in vals.items():
                if key not in self._columns and key.split('_')[0]!='foto':
                    self.more_datas[id][key]=value
        return ret

    def create(self, cr, uid, vals, context=None):
#        print datetime.today(),"update_seance_attendance: create: vals, context : ", vals, context
        if not context: context={}
        vals.setdefault('seance_id', context.get('seance_id',context.get('active_id',None)))
        ret=super(update_seance_attendance, self).create(cr, uid, vals, context=context)
        seance=self.pool.get('school.seance').browse(cr, self.user_manager_id, vals['seance_id'])
        item={}

        state_codes=set()
        contact_ids=set()
        fields_more=self.fields_get(cr, uid, context={'seance_id': seance.id})
        for field in fields_more:
            parts=field.split('_')
            if len(parts)>2 and parts[0]==PREFIXE:
                contact_ids.add(int(parts[-1]))
                state_codes.add('_'.join(parts[1:-1]))
        state_ids=self.pool.get('contact.annotation.state').search(cr, self.user_manager_id, [('code','in',list(state_codes))])
        sr2_ids=self.pool.get('contact.state_range2').search(cr, self.user_manager_id, [('lstate.id','in',state_ids),('contact_id.id','in',list(contact_ids)),('datetime_from','<',seance.date_to),('datetime_to','>',seance.date)], context={
            'wizard_seance_attendance_state_ids': state_ids,
            'wizard_seance_attendance_contact_ids': list(contact_ids)})
        
        for field in fields_more:
            if field.split('_')[0]==PREFIXE:
                item[field]=False

        for sr2 in self.pool.get('contact.state_range2').browse(cr, uid, sr2_ids):
            item[PREFIXE+'_'+str(sr2.lstate.code)+'_'+str(sr2.contact_id.id)]=True

        self.more_datas[ret]=item
        return ret

    def unlink(self, cr, uid, ids, context=None):
        for id in ids: del self.more_datas[id]
        return super(update_seance_attendance, self).unlink(cr, uid, ids, context=context)

    def act_cancel(self, cr, uid, ids, context=None):
        return {'type': 'ir.actions.act_window_close'}

    def _get_informer(self, cr, uid, seance):
        ret=False
        for teacher_data in seance.teachers:
            if uid==teacher_data.teacher_id.teacher_user_id.id: ret=teacher_data.teacher_id.id
        # Program substitution
        if seance.room_id and not ret:
            cr.execute("select p.id from school_teacher_seance as ts, school_seance as s, res_partner as p, school_program_substitutions as ps where ts.seance_id=s.id and ts.teacher_id=p.id and p.user_id=%s and s.classe_id=ps.classe_id and ps.room_id=%s and s.date>=%s and s.date_to<=%s",(uid, seance.room_id.id, seance.date, seance.date_to))
            for (teacher_id,) in cr.fetchall(): ret=teacher_id
        return ret

    def act_update(self, cr, uid, ids, context=None):
        usa_obj=self
        uid2=self.user_manager_id
        seance_ids=[]
        seance_ids_to_close=[]
        for item in usa_obj.read(cr, uid2, ids, [], context=context):
            seance=self.pool.get('school.seance').browse(cr, uid, item['seance_id'])
            informer=self._get_informer(cr, uid, seance)
            if not informer:
                if uid in self.pool.get('res.users').search(cr, uid2, [('groups_id.name','=','Wizard group')]):
                    informer_partners=self.pool.get('res.partner').search(cr, uid2, [('user_id','=',uid)])
                    if informer_partners: informer=informer_partners[0]
                else:
                    raise orm.except_orm(_('Not granted!'),
                            _('Only seance teachers or Wizard Group members could register attendance.'))
            actual_id=usa_obj.create(cr, uid, {'seance_id': item['seance_id']})
            item_actual=usa_obj.read(cr, uid, [actual_id], [], context=context)[0]
            contact_ids=set()
            contact_ids_with_absence_codes=[]
            for field in item.keys():
                parts=field.split('_')
                if len(parts)>2 and parts[0]==PREFIXE and field in item_actual:
                    state_code='_'.join(parts[1:-1])
                    contact_id=int(parts[-1])
                    contact_ids.add(contact_id)
                    if ABSENCE_CODE_STATE==state_code and item[field]: contact_ids_with_absence_codes.append(contact_id)
                    if item_actual[field]!=item[field]:
                        af_obj=self.pool.get('school_attendance.aux.fields_view')
                        ids2=af_obj.search(cr, uid2, [('check_state','=',state_code)])
                        anno=False
                        if item[field]:
                            anno=af_obj.read(cr, uid2, ids2, ['anno_add'])[0]['anno_add'][0]
                        else:
                            anno=af_obj.read(cr, uid2, ids2, ['anno_subs'])[0]['anno_subs'][0]
                        self.pool.get('contact.annotation').create(cr, uid2, {'anno_type': anno, 'contact_id': contact_id, 'valid_from': seance.date, 'valid_to': seance.date_to, 'informer': informer, 'comment': item.get('comment_%s' % contact_id,False)})

            self.pool.get('school.presence').unlink(cr, uid2, self.pool.get('school.presence').search(cr, uid2, [('seance_id','=',seance.id)]))
            if datetime.strptime(seance.date,'%Y-%m-%d %H:%M:%S')<datetime.today():
                seance_ids_to_close.append(seance.id)
                for contact_id in contact_ids.difference(contact_ids_with_absence_codes):
                    self.pool.get('school.presence').create(cr, uid2, {'seance_id': seance.id, 'student': contact_id, 'user_id': uid})
            seance_ids.append(seance.id)

        cr.execute("UPDATE school_seance SET last_attendance_update=%s,user_attendance_update=%s WHERE id in %s", ( datetime.today().strftime('%Y-%m-%d %H:%M:%S'), uid, tuple(seance_ids) ) )
        if seance_ids_to_close:
            cr.execute("UPDATE school_seance SET state='closed' WHERE id in %s", ( tuple(seance_ids_to_close), ) )
        return {'type': 'ir.actions.act_window_close'}

update_seance_attendance()

class school_seance(osv.osv):
    _name = 'school.seance'
    _inherit = 'school.seance'

    _columns = {
        'presences' : fields.one2many('school.presence', 'seance_id', 'Presences', ),
        'last_attendance_update' : fields.datetime('Last attendance update',select=1,),
        'user_attendance_update' : fields.many2one('res.users','User',help="Who update the attendance"),
    }

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context: context={}
        context2=dict(context)
        if context.get('seances_to_close',False):
            days=self.pool.get('school.params').get_value(cr, 1, 'seances_to_close_days_before', context=context) or 7
            query="""SELECT s.id FROM school_seance AS s LEFT JOIN school_teacher_seance AS ts ON s.id=ts.seance_id LEFT JOIN res_partner AS p ON ts.teacher_id=p.id LEFT JOIN school_classe AS c ON s.classe_id=c.id LEFT JOIN school_course AS co ON c.course_id=co.id WHERE NOT co.special AND p.user_id=%s AND s.last_attendance_update IS NULL AND s.date BETWEEN now()-INTERVAL '%s DAYS' AND now()"""
            cr.execute(query, (uid,days))
            context2.pop('seances_to_close', None)
            args=[('id','in',[x for (x,) in cr.fetchall()])]
        ret=super(school_seance, self).search(cr, uid, args, offset=offset, limit=limit, order=order, context=context2, count=count)
        return ret

school_seance()

class school_presence(osv.osv):
    _name = 'school.presence'

    def _get_name(self, cr, uid, ids, field_name, arg, context=None):
        ret={}
        for presence_data in self.read(cr, uid, ids, ['student','seance_id']):
            ret[presence_data['id']]=presence_data['student'][1]+' - '+presence_data['seance_id'][1]
        return ret
        
    _columns = {
        'name' : fields.function(_get_name, method=True, type='char', size=100, string='Name', store={'school.presence': (lambda self,cr,uid,ids,context: ids,['student','seance_id'],10)}),
        'student' : fields.many2one('res.partner.contact', 'Student',),
        'seance_id' : fields.many2one('school.seance', 'Seance',),
        'user_id' :fields.many2one('res.users', 'Supervisor',readonly=False,),
        'student_name' : fields.related('student','name',type='char',string='Name',),
        'date' : fields.related('seance_id','date',type='datetime',string='Date',),
    }

    _defaults = {
        'user_id' : lambda self,cr,uid,context: uid,
    }

    _order = 'name'
school_presence()

class contact_annotation(osv.osv):
    _name = 'contact.annotation'
    _inherit = 'contact.annotation'


    def get_contacts(self, cr , uid, context=None):
        if not context: context={}
        if 'wizard_seance_attendance_contact_ids' not in context: return super(contact_annotation, self).get_contacts(cr , uid, context=context)
        return context['wizard_seance_attendance_contact_ids']

    def get_states(self, cr, uid, context=None):
        if not context: context={}
        if 'wizard_seance_attendance_state_ids' not in context: return super(contact_annotation, self).get_states(cr , uid, context=context)
        return context['wizard_seance_attendance_state_ids']

    def search(self, cr , uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context: context={}
        if 'domain_contact_id' in context: args+=[('contact_id','=',context['domain_contact_id'])]
        if 'domain_valid_from' in context: args+=[('valid_to','>',context['domain_valid_from'])]
        if 'domain_valid_to' in context: args+=[('valid_from','<',context['domain_valid_to'])]
        return super(contact_annotation, self).search(cr , uid, args, offset=offset, limit=limit, order=order, context=context, count=count)

    def default_get(self, cr, uid, fields, context=None):
        if not context: context={}
        ret=super(contact_annotation, self).default_get(cr, uid, fields, context=context)
        if 'default_contact_id' in context: ret['contact_id']=context['default_contact_id']
        if 'default_valid_from' in context: ret['valid_from']=context['default_valid_from']
        if 'default_valid_to' in context: ret['valid_to']=context['default_valid_to']
        return ret


contact_annotation()

class contact_state_range(osv.osv):
    _name='contact.state_range'
    _inherit='contact.state_range'

    def _seances_for_state_range(self, cr, uid, ids, field_name, args, context=None):
        ret={}
        for item in self.browse(cr, uid, ids, context=context):
            ret[item.id]=[]
            part_ids=self.pool.get('groups.participation').search(cr, uid, [('participant','=',item.contact_id.id)])
            if part_ids:
                for intervals in self.pool.get('groups.participation').get_seances(cr, uid, part_ids, item.datetime_from, item.datetime_to).values():
                    for interval in intervals:
                        if interval[0] not in ret[item.id]: ret[item.id].append(interval[0])
        return ret

    _columns={
        'seance_ids': fields.function(_seances_for_state_range, type='one2many', obj='school.seance', method=True, string='Seances', ),
    }
contact_state_range()

class open_wizard_justify(wizard.interface):
    def _open_wizard_justify(self, cr, uid, data, context=None):
        if not context: context={}
        mod_obj = pooler.get_pool(cr.dbname).get('ir.model.data')
        act_obj = pooler.get_pool(cr.dbname).get('ir.actions.act_window')

        result = mod_obj._get_id(cr, uid, 'contact_annotation', 'contact_planning_change_states_list_act')
        id = mod_obj.read(cr, uid, [result], ['res_id'])[0]['res_id']
        result = act_obj.read(cr, uid, [id], [])[0]

        result1 = mod_obj._get_id(cr, uid, 'school_attendance', 'contact_planning_justify_form_view')
        view_id = mod_obj.read(cr, uid, [result1], ['res_id'])[0]['res_id']
        result['views'] = [(view_id,'form')]

        obj=pooler.get_pool(cr.dbname).get('contact.planning_change_states')
        vals=obj.default_get(cr, uid, obj.fields_get(cr, uid, context=context).keys(), context={'active_ids': data['ids']})
        vals['anno_type']=pooler.get_pool(cr.dbname).get('contact.type_annotation').search(cr, uid, [('code','=','JUSTI')])[0]
        res_id=obj.create(cr, uid, vals, context={'active_ids': data['ids']})

        result['res_id'] = res_id
        return result

    states = {
          'init': {
                'actions': [],
                'result': {'type': 'action',
                           'action': _open_wizard_justify,
                           'state':'end'}
          },
    }

open_wizard_justify('contact_open_justify_states')

class open_wizard_attendance(wizard.interface):
    def _open_wizard_attendance(self, cr, uid, data, context=None):
#        print datetime.today(),"open_wizard_attendance: _open_wizard_attendance: data, context : ", data, context
        if not context: context={}
        mod_obj = pooler.get_pool(cr.dbname).get('ir.model.data')
        act_obj = pooler.get_pool(cr.dbname).get('ir.actions.act_window')

        result = mod_obj._get_id(cr, uid, 'school_attendance', 'action_wizard_update_seance_attendance')
        id = mod_obj.read(cr, uid, [result], ['res_id'])[0]['res_id']
        result = act_obj.read(cr, uid, [id], [])[0]
        views={}
        for (view_id,view_mode) in result['views']: views[view_mode]=view_id
        result['views'] = [(views['form'],'form')]

        obj=pooler.get_pool(cr.dbname).get('wizard.school_attendance.update_seance_attendance')
        res_id=obj.create(cr, uid, {'seance_id': data['id']})

        result['res_id'] = res_id
        result['res_ids'] = [res_id]
        if context.get('client','')=='web':
            result['target'] = 'new'
#        print datetime.today(),"open_wizard_attendance: _open_wizard_attendance: data, context : ", data, context
        return result

    states = {
          'init': {
                'actions': [],
                'result': {'type': 'action',
                           'action': _open_wizard_attendance,
                           'state':'end'}
          },
    }

open_wizard_attendance('school_attendance_open_wizard')

class school_panel(osv.osv_memory):
    _name = 'school.panel'

    _columns = {
        'name': fields.char('Name',size=200,),
        'sessions_per_tancar': fields.integer('Per Tancar',),
        'estats_per_justificar': fields.integer('Per justificar',),
        }

    _defaults = {
        'sessions_per_tancar': lambda self,cr,uid,context: 0,
        'estats_per_justificar': lambda self,cr,uid,context: 0,
    }

school_panel()

class school_change_seance_teacher(osv.osv_memory):
    _name = 'school.change_seance_teacher'

    _columns = {
        'name': fields.char('Name',size=200,),
        'sessions_per_tancar': fields.integer('Per Tancar',),
        'estats_per_justificar': fields.integer('Per justificar',),
        }

    _defaults = {
        'sessions_per_tancar': lambda self,cr,uid,context: 0,
        'estats_per_justificar': lambda self,cr,uid,context: 0,
    }

school_panel()

class school_params(osv.osv):
    _name = 'school.params'
    _inherit = 'school.params'

    _columns = {
        'seances_to_close_days_before' : fields.integer('Days Before at Seances to Close'),
    }

school_params()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

