# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#      Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#    Copyright (C) 2011 Zikzakmedia S.L. (http://www.zikzakmedia.com)
#      Jesús Martín Jiménez <jmargin@zikzakmedia.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program. If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import osv, fields, orm
from datetime import datetime, date, timedelta
from tools.translate import _
from timed_groups.timed_groups import _MIN_DATETIME,_MAX_DATETIME

class res_partner(osv.osv):
    _name  =  "res.partner"
    _inherit  =  "res.partner"
    _columns  =  {
        'doc_type': fields.char('Doc type', size = 1),
    }

res_partner()

class school_school(osv.osv):
    _name = 'school.school'
school_school()

class res_users(osv.osv):
    _name  =  "res.users"
    _inherit  =  "res.users"
    _columns  =  {
        'school_id': fields.many2one('school.school','School', select = 1, help = 'School that is related to the current user'),
    }

res_users()

class res_partner_contact(osv.osv):
    _name  =  "res.partner.contact"
    _inherit  =  "res.partner.contact"

    _columns  =  {
        'doc_type': fields.char('Doc type', size = 1, ),
        'vat': fields.char('VAT',size = 32 ),
        'gender': fields.selection([('M','Man'),('W','Woman')],'Gender'),
    }

res_partner_contact()

class school_student(osv.osv):
    _name = "school.student"
    _inherits = {'res.partner.contact' : 'contact_id',}

    _columns = {
        'contact_id' : fields.many2one('res.partner.contact','Contact',),
    }

school_student()

class school_teacher(osv.osv):
    _name = "school.teacher"
    _rec_name = 'teacher_name'

    def _get_partner_data(self, cr, uid, ids, field_name, arg, context=None):
        ret={}
        for item in self.browse(cr, uid, ids):
            ret[item.id] = {}
            if item.partner_id:
                if item.partner_id.user_id:
                    ret[item.id]['teacher_user_id']=item.partner_id.user_id.id
                else:
                    ret[item.id]['teacher_user_id']=False
                ret[item.id]['teacher_name']=item.partner_id.name
            else:
                ret[item.id]={'teacher_user_id': False, 'teacher_name': False, }
        return ret

    _columns = {
        'partner_id' : fields.many2one('res.partner','Partner',ondelete='cascade',),
        'teacher_user_id' : fields.function(_get_partner_data,obj='res.users',type='many2one',method=True,multi='data',store={
            'res.partner': (lambda self,cr,uid,ids,context: self.pool.get('school.teacher').search(cr, uid, [('partner_id','in',ids)]),['user_id'],10),
            'school.teacher': (lambda self,cr,uid,ids,context: ids,['partner_id'],10),
        }),
        'teacher_name' : fields.function(_get_partner_data,type='char',size=200,string='Name',method=True,multi='data',store={
            'res.partner': (lambda self,cr,uid,ids,context: self.pool.get('school.teacher').search(cr, uid, [('partner_id','in',ids)]),['name'],10),
            'school.teacher': (lambda self,cr,uid,ids,context: ids,['partner_id'],10),
        }),
    }

school_teacher()

class school_course(osv.osv):
    _name  =  'school.course'
    _columns  =  {
        'name' : fields.char('Name', size = 100, required = True, select = 1, help = "The course name."),
        'code' : fields.char('Code', size = 10, required = True, select = 1, help = "The course code."),
        'special' : fields.boolean('Special', select = 1, help = "If the course is to put eat outher." ),
        'active' : fields.boolean('Active', select = 1),
        'min_hours' : fields.integer('Min Hours'),
    }

    _defaults  =  {
        'active' : lambda *a: 1,
        'min_hours' : lambda *a : 1,
    }

    _order  =  'name'

school_course()

class school_offer(osv.osv):
    _name = 'school.offer'
school_offer()


class school_course_line(osv.osv):
    _name  =  'school.course.line'
    _columns  =  {
        'course_id' : fields.many2one('school.course', 'Course',ondelete = 'restrict', required = True, select = 1, help = "The course."),
        'offer_id': fields.many2one('school.offer', 'Offer', ondelete = 'restrict', required = True, select = 1, help = "The offer."),
        'weekly_hours' : fields.integer('Weekly hours'),
        'needly' : fields.selection(
            [
                ('required', 'Required'),
                ('optional', 'Optional'),
            ],
            'Needly',
            readonly = False,
            help = "The need of the course into the offer.",
            required = True,
        ),
    }

    _order  =  'needly'
    _defaults  =  {
        'needly' : lambda *a: 'required',
    }

    _sql_constraints  =  [('course_unique_by_offer','unique(course_id,offer_id)','Course couldn''t repeat in a offer')]

    def name_get(self, cr, uid, ids):
        ret = {}
        for id in ids: ret[id] = ''
        query = """
SELECT scl.id, sc.name || ' (' || so.name || ')'
FROM school_course_line as scl
 INNER JOIN school_offer AS so ON scl.offer_id = so.id
 INNER JOIN school_course AS sc ON scl.course_id = sc.id
WHERE scl.id IN %s"""
        cr.execute(query,(tuple(ids),))
        for (id,name) in cr.fetchall(): ret[id] = name
        # @type ret dict
        return ret.items()

    def name_search(self, cr, uid, name = '', args = [], operator = 'ilike', context = None, limit = 80):
        res = []
        query = """
SELECT scl.id, sc.name || ' (' || so.name || ')'
FROM school_course_line as scl
 INNER JOIN school_offer AS so ON scl.offer_id = so.id
 INNER JOIN school_course AS sc ON scl.course_id = sc.id
WHERE sc.name || ' (' || so.name || ')' ilike '%%%s%%'"""
        cr.execute(query,(name,))
        data = cr.fetchall()
        ids = zip(data)[0]
        ids2 = self.search(cr, uid, [('id','in',ids)]+args, limit = limit, context = context)
        for tupla in data:
            if tupla[0] in ids2: res.append(tupla)
        return res

school_course_line()


class school_offer(osv.osv):
    _name = 'school.offer'

    _columns = {
        'name' : fields.char('Name', size=32, required=True, select=1, help="The offer name"),
        'courses_lines' : fields.one2many('school.course.line','offer_id','Courses',help='List of the courses in the offer'),
        'active' : fields.boolean('Active', select=2,),
        'session_ids' : fields.one2many('school.session','offer',string='Sessions',),
    }

    _defaults = {
        'active' : lambda *a: 1,
    }

    _order = 'name'

school_offer()


class school_session(osv.osv):
    _name = 'school.session'

    
    _columns = {
        'name' : fields.char('Name', size=32, required=True, select=1, help="The session name"),
        'offer' : fields.many2one('school.offer','Offer',select=1,help="The offer relationed with the item",),
        'date_from' : fields.date('Date from', required=True,),
        'date_to' : fields.date('Date to', required=True,),
        'participants' : fields.one2many('groups.participation','session_id', ),
        'active' : fields.boolean('Active', select=2,),
        'school_id': fields.many2one('school.school', 'School', select = 1, help = 'School that is related to the current session.'),
    }

    _defaults = {
        'active' : lambda *a: 1,
    }

    _sql_constraints = [
        ('dates_ok','CHECK (date_to>=date_from)','Date to minor than date from'),
        ('name_uniq', 'unique (name)', 'The Name of the Session must be unique !')
    ]

    _order = 'name'

    def copy(self, cr, uid, id, default = {}, context = None):
        session_name = self.read(cr, uid, [id], ['name'])[0]['name']
        default.update({
            'name': _('%s (copy)') % session_name,
        })
        return super(school_session, self).copy(cr, uid, id, default, context)

school_session()

class school_holidays_calendar(osv.osv):
    _name = 'school.holidays_calendar'
school_holidays_calendar()

class school_holiday_interval(osv.osv):
    _name = 'school.holiday_interval'
    _columns = {
        'name' : fields.char('Name', size=30,),
        'date_from' : fields.date('Date from', select=1, required=True,),
        'date_to' : fields.date('Date to', select=1, required=True,),
        'calendars' : fields.many2many('school.holidays_calendar','school_holidays_calendar_intervals_rel','days_id','cal_id',string="Calendars"),
    }

    def _check_date_from_to(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids[0], context=context)
        return obj.date_from <= obj.date_to

    _constraints = [
        (_check_date_from_to, "Please, check the start date !", ['date_from', 'date_to']),
    ]

    _sql_constraints = [
        ('uniq_holiday_interval', 'unique(name)', 'The holiday interval of the year must be unique !'),
    ]

school_holiday_interval()

class school_holidays_calendar(osv.osv):
    _name = 'school.holidays_calendar'
    _columns = {
        'name' : fields.char('Name', size=30,),
        'holidays' : fields.many2many('school.holiday_interval','school_holidays_calendar_intervals_rel','cal_id','days_id',string="Holidays"),
        'school_id': fields.many2one('school.school', 'School', select = 1, help = 'School that is related to the current calendar.'),
    }
    
    _sql_constraints = [
        ('uniq_holiday_calendar', 'unique(name)', 'The holiday calendar must be unique !'),
    ]

school_holidays_calendar()

class school_school(osv.osv):
    _name = 'school.school'
    _columns = {
        'name': fields.char('Name', size = 30, required = True, select = 1),
        'partner_id': fields.many2one('res.partner', 'Partner', required = True, select = 1, help = 'Partner that is related to the current school.'),
        'user_ids': fields.one2many('res.users', 'school_id', string = 'Users', help = 'Users of this school.'),
        'calendar_ids': fields.one2many('school.holidays_calendar', 'school_id', string = 'Calendars', help = 'Calendars of this school.'),
        'session_ids': fields.one2many('school.session', 'school_id', string = 'Sessions', help = 'Sessions of this school.'),
    }

school_school()

class school_params(osv.osv):
    _name = 'school.params'
    _description = "To inherit and to add module parameters."

    _columns = {
        'name' : fields.char('',size=6,readonly=True,),
    }

    def default_get(self, cr, uid, fields=None, context=None):
        ids=self.search(cr, uid, [('id','!=',0)])
        if ids:
            return super(school_params, self).read(cr, uid, ids, fields or [], context=context)[0]
        ret={}.fromkeys(fields or [], False)
        ret['name']=''
        return ret

    def create(self, cr, uid, vals, context=None):
        ids=self.search(cr, uid, [('id','!=',0)])
        if ids:
            return super(school_params, self).write(cr, uid, ids, vals, context=context)
        return super(school_params, self).create(cr, uid, vals, context=context)

    def get_value(self, cr, uid, param, context=None):
        ret=False
        for data in self.read(cr, uid, self.search(cr, uid, [('id','!=',0)]), [param]):
            ret=data[param]
        return ret
school_params()



# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
