# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields

def mean(numberList):
    if len(numberList) == 0:
        return float('nan')
    floatNums = [float(x) for x in numberList]
    return sum(floatNums) / len(numberList)


class school_evaluation(osv.osv):
    _name = 'school.evaluation'
    _columns = {
        'participation_id' : fields.many2one('groups.participation','Participation',ondelete='restrict',required=True,),
        'course_id' : fields.many2one('school.course','Course',ondelete='restrict',required=True,),
        'level' : fields.float('Level',help='Evaluation level (0-10)',required=True,),
        'teacher_id' : fields.many2one('school.teacher','Teacher',),
    }
    _sql_constraints = [('level_correct','CHECK (level BETWEEN 0 AND 10)','Level must be between 0 and 10')]
school_evaluation()

class school_participation_course_data(osv.osv):
    _name = 'school.participation_course_data'
    _inherit = 'school.participation_course_data'

    def _get_evaluation(self, cr, uid, ids, field_name, arg, context=None):
        ret=dict.fromkeys(ids,False)
        query="""
        SELECT pcd.id,AVG(e.level) AS level FROM school_participation_course_data AS pcd INNER JOIN
        school_evaluation AS e ON pcd.participation_id=e.participation_id AND pcd.course_id=e.course_id
        WHERE pcd.id IN %s GROUP BY pcd.id"""
        cr.execute(query, (tuple(ids),))
        for (id,level) in cr.fetchall(): ret[id]=level
        return ret

    def _search_pcd_by_evaluation(self, cr, uid, ids, context=None):
        query="""
        SELECT pcd.id FROM school_evaluation AS e INNER JOIN
            school_participation_course_data AS pcd ON e.participation_id=pcd.participation_id AND e.course_id=pcd.course_id
        WHERE e.id IN %s"""
        cr.execute(query, (tuple(ids),))
        return [x for (x,) in cr.fetchall()]

    _columns = {
        'evaluation' : fields.function(_get_evaluation, method=True, string='Evaluation', type='float', store={
                'school.evaluation': ( _search_pcd_by_evaluation, ['participation_id','course_id','level'], 10),
                },),
    }
school_participation_course_data()

class groups_participation(osv.osv):
    _name = 'groups.participation'
    _inherit = 'groups.participation'

    def _get_evaluation(self, cr, uid, ids, field_name, arg, context=None):
        ret=dict.fromkeys(ids,False)
        query="""
        SELECT pcd.id,AVG(evaluation) AS level FROM school_participation_course_data AS pcd
        WHERE pcd.participation_id IN %s GROUP BY pcd.id"""
        cr.execute(query, (tuple(ids),))
        for (id,level) in cr.fetchall(): ret[id]=level
        return ret

    def _search_part_by_evaluation(self, cr, uid, ids, context=None):
        query="""SELECT e.participation_id FROM school_evaluation AS e WHERE e.id IN %s"""
        cr.execute(query, (tuple(ids),))
        return [x for (x,) in cr.fetchall()]

    _columns = {
        'evaluation' : fields.function(_get_evaluation, method=True, string='Evaluation', type='float', store={
                'school.evaluation': ( _search_part_by_evaluation, ['participation_id','course_id','level'], 20),
                },),
    }
    
groups_participation()

class school_completed_offer(osv.osv):
    _name = 'school.completed_offer'

    _columns = {
        'offer_id' : fields.many2one('school.offer','Offer',required=True,ondelete='restrict',),
        'contact_id' : fields.many2one('res.partner.contact','Student',required=True,ondelete='restrict',),
        'level' : fields.integer('Level',),
        'source' : fields.selection([('internal','Internal'),('external','External')],'Source',help='Source of the information',),
    }
school_completed_offer()

class res_partner_contact(osv.osv):
    _name = 'res.partner.contact'
    _inherit = 'res.partner.contact'

    def _check_completed_offers(self, cr, uid, ids, context=None):
        obj=self.pool.get('school.completed_offer')

        # Unlink the computed offers for the contacts
        ids2=obj.search(cr, uid, [('contact_id','in',ids),('source','=','internal')], context=context)
        obj.unlink(cr, uid, ids2, context=context)

        # Fill the level matrix for contact and approved course
        query="""SELECT p.participant,pcd.course_id,pcd.evaluation FROM groups_participation AS p INNER JOIN
            school_participation_course_data AS pcd ON pcd.participation_id=p.id
            WHERE pcd.evaluation>5 AND p.participant IN %s"""
        cr.execute(query,(tuple(ids),))
        courses_by_contact={}
        all_course_ids=set()
        for (contact_id,course_id,level) in cr.fetchall():
            all_course_ids.add(course_id)
            if contact_id not in courses_by_contact:
                courses_by_contact[contact_id]={}
            dicci=courses_by_contact[contact_id]
            dicci[course_id]=course_id in dicci and max(dicci[course_id],level) or level

        # Fill de courses set by offer
        courses_by_offer={}
        query="SELECT DISTINCT offer_id FROM school_course_line WHERE course_id IN %s"
        cr.execute(query, (tuple(all_course_ids),))
        offer_ids=[x for (x,) in cr.fetchall()]
        query="SELECT DISTINCT offer_id,course_id FROM school_course_line WHERE offer_id IN %s"
        cr.execute(query, (tuple(offer_ids),))
        for (offer_id,course_id) in cr.fetchall():
            if offer_id not in courses_by_offer:
                courses_by_offer[offer_id]=set()
            courses_by_offer[offer_id].add(course_id)

        # Create offers completed
        for (contact_id,courses) in courses_by_contact.items():
            for (offer_id,course_ids_set) in courses_by_offer:
                contact_courses_set=set(courses.keys())
                if contact_courses_set >= course_ids_set:
                    obj.create(cr, uid, {'offer_id': offer_id,
                        'contact_id': contact_id,
                        'level': mean(courses[x] for x in course_ids_set),
                        'source': 'internal',
                        })

    _columns = {
        'completed_offer_ids' : fields.one2many('school.completed_offer','contact_id',string='Completed Offers',),
    }

res_partner_contact()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
