# -*- encoding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school modules
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import pooler
from datetime import datetime, timedelta
import copy
# import xml.dom.minidom

#FIXME: we should use toxml

ADMIN_ID=1
_DISCARD_ABSENCES_SAME_TIME_CONTEXT_KEY='DISCARD_ABSENCES_SAME_TIME'

def get_part_data(cr, uid, ids, datetime_from, datetime_to, context=None):
    if not context: context=None
    result={}

    pool=pooler.get_pool(cr.dbname)
    obj_part=pool.get('groups.participation')

    contact_by_part={}
    part_by_contact={}
    for x in obj_part.read(cr, uid, ids, ['participant']):
        part_by_contact[x['participant'][0]]=x['id']
        contact_by_part[x['id']]=x['participant'][0]

    state_range_ids=pool.get('contact.state_range2').search(cr, ADMIN_ID, [('contact_id.id','in',part_by_contact.keys()),('lstate.code','in',('unjustified','absent','delayed','expulsion','incidence')),('datetime_to','>',datetime_from),('datetime_from','<',datetime_to)], context=None)
    state_ranges=pool.get('contact.state_range2').browse(cr, ADMIN_ID, state_range_ids)
    part_seances=obj_part.get_seances(cr, uid, ids, datetime_from=datetime_from, datetime_to=datetime_to )
    all_seances={}
    for (part_id, seances_data) in part_seances.items():
        for (seance_id,seance_datetime_from,seance_datetime_to,op) in seances_data: all_seances[seance_id]={}
    for seance in pool.get('school.seance').browse(cr, uid, all_seances.keys()):
        all_seances[seance.id]={'course_id': seance.classe_id.course_id.id, 'state': seance.state, 'teacher_ids': []}
        for teacher_data in seance.teachers: all_seances[seance.id]['teacher_ids'].append(teacher_data.teacher_id.id)

    stat_zero={'time_seances': timedelta(),
        'id_sessions':set(),
        'absent_time': timedelta(),
        'absent_time_j': timedelta(),
        'absent': 0,
        'absent_j': 0,
        'delayed': 0,
        'delayed_j': 0,
        'incidence': 0,
        'expulsion': 0, }
    
    discard_absences_same_time=[]
    for (part_id, seances_data) in part_seances.items():
        if part_id not in result: result[part_id]={'by_date': {}, 'by_teacher': {}, 'by_course': {}, 'total': copy.deepcopy(stat_zero),}
        
        for (seance_id,seance_datetime_from,seance_datetime_to,op) in seances_data:
            key_absences_same_time=(part_id,seance_datetime_from,seance_datetime_to)
            if context.get(_DISCARD_ABSENCES_SAME_TIME_CONTEXT_KEY,False) and key_absences_same_time in discard_absences_same_time:
                continue
            discard_absences_same_time.append(key_absences_same_time)
            ldate=seance_datetime_from[0:10]
            course_id=all_seances[seance_id]['course_id']
            teacher_ids=all_seances[seance_id]['teacher_ids']
            seance_state=all_seances[seance_id]['state']

            seance_dtf=datetime.strptime(seance_datetime_from,'%Y-%m-%d %H:%M:%S')
            seance_dtt=datetime.strptime(seance_datetime_to,'%Y-%m-%d %H:%M:%S')
            part_seance_stat={'time_seances': (seance_dtt-seance_dtf) }
            part_contact_id=contact_by_part[part_id]
            for item in state_ranges:
                if item.contact_id.id==part_contact_id:
                    range_state_dtf=datetime.strptime(item.datetime_from,'%Y-%m-%d %H:%M:%S')
                    range_state_dtt=datetime.strptime(item.datetime_to,'%Y-%m-%d %H:%M:%S')
                    if seance_dtt>range_state_dtf and seance_dtf<range_state_dtt:
                        if item.lstate.code not in part_seance_stat: part_seance_stat[item.lstate.code]=timedelta()
                        part_seance_stat[item.lstate.code]+=(min(range_state_dtt,seance_dtt)-max(range_state_dtf,seance_dtf))

            def update_stat(stat_global, one_stat, seance_id):
                stat_global['id_sessions'].add(seance_id)
                stat_global['time_seances'] += one_stat['time_seances']
                if 'absent' in one_stat:
                    stat_global['absent'] += 1
                    if 'unjustified' in one_stat:
                        stat_global['absent_j'] += 1
                    stat_global['absent_time'] += one_stat['absent']
                    if 'unjustified' in one_stat:
                        stat_global['absent_time_j'] += max(timedelta(),min(one_stat['absent'],one_stat['unjustified']))
                if 'delayed' in one_stat:
                    stat_global['delayed'] += 1
                    if 'unjustified' in one_stat:
                        stat_global['delayed_j'] += 1
                if 'incidence' in one_stat:
                    stat_global['incidence'] += 1
                if 'expulsion' in one_stat:
                    stat_global['expulsion'] += 1

            update_stat(result[part_id]['total'], part_seance_stat, seance_id)
            claus=[('by_date',ldate),('by_teacher',teacher_ids),('by_course',course_id)]
            for (key1,key2) in claus:
                list2=key2
                if type(key2)!=type([]): list2=[list2]
                for key2 in list2:
                    if key2 not in result[part_id][key1]:
                        result[part_id][key1][key2]=copy.deepcopy(stat_zero)
                    update_stat(result[part_id][key1][key2], part_seance_stat, seance_id)

    query="""
SELECT c.name,c.first_name,p.id,MAX(t.name) as tutor_name
FROM groups_participation as p
INNER JOIN res_partner_contact as c ON p.participant=c.id
LEFT JOIN contact_partner_function AS cpf ON cpf.contact_id=c.id
LEFT JOIN res_partner AS t ON cpf.partner_id=t.id
WHERE p.id in %s
GROUP BY c.name,c.first_name,p.id
ORDER BY c.name,c.first_name
    """
    cr.execute(query, (tuple(ids),))
    for (name,first_name,id_part,tutor_name) in cr.fetchall():
        if id_part not in result: result[id_part]={'by_date': {}, 'by_teacher': {}, 'by_course': {}, 'total': copy.deepcopy(stat_zero), }
        result[id_part].update( {'name': name, 'first_name': first_name, 'tutor_name': tutor_name,} )
    return result


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

