# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#      Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#    Copyright (C) 2011 Zikzakmedia S.L. (http://www.zikzakmedia.com)
#      Jesús Martín Jiménez <jmargin@zikzakmedia.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program. If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import osv, fields, orm
from datetime import datetime, date, timedelta
from tools.translate import _
from timed_groups.timed_groups import _MIN_DATETIME,_MAX_DATETIME

class school_session(osv.osv):
    _name = 'school.session'
    _inherit = 'school.session'
    
    def unlink(self, cr, uid, ids, context=None):
        if not context: context={}
        # @type context dict
        if not context.get('entorn_superconfirmat', False):
            raise orm.except_orm('Not granted!','Session deletion must to do throught wizard.')
        return super(school_session, self).unlink(cr, uid, ids, context=context)

school_session()

class school_building(osv.osv):
    _name = 'school.building'
school_building()

class school_room(osv.osv):
    _name = 'school.room'
    _columns = {
        'name' : fields.char('Name', size=32, required=True, select=1, help="The room name"),
        'seance_ids' : fields.one2many('school.seance','room_id','Seances',select=1),
        'capacity' : fields.integer('Capacity',),
        'building_id' : fields.many2one('school.building',string='Building',ondelete='set null',),
    }
    _defaults = {
        'capacity' : lambda *a : 50,
    }
    _order = 'name'
school_room()

class school_building(osv.osv):
    _name = 'school.building'
    _columns = {
        'name' : fields.char('Name', size=32, required=True, select=1, help="The building name"),
        'rooms' : fields.one2many('school.room','building_id',string='Rooms',),
    }
    _order = 'name'
school_building()

class school_classe(osv.osv):
    _name = 'school.classe'
        
    _columns = {
        'name' : fields.char('Name',size=200,select=1,help="The course's name",),
        'course_id' : fields.many2one('school.course','Course',required=True,readonly=False,help="The course",),
        'seance_ids' : fields.one2many('school.seance','classe_id',select=True,help='List of the seances for a class.',),
        'date_from': fields.date('Date From',required=True,),
        'date_to': fields.date('Date To',required=True,),
        'group_id': fields.many2one('groups.group','Group',select=1,help='A group linked',readonly=False,ondelete='restrict',),
        'wrong_seances' : fields.boolean('Wrong Seances',select=1),
        'holidays_calendar_id' : fields.many2one('school.holidays_calendar',string="Holidays Calendar",ondelete="set null",),
    }

    _order = 'name'
    
    def _constraint_no_seances_closed_outher_dates(self, cr, uid, ids):
        sql="""
        SELECT s.id
        FROM school_classe AS c
        INNER JOIN school_seance AS s ON s.classe_id=c.id
        WHERE s.state IN ('opened','closed') AND s.date NOT BETWEEN c.date_from AND c.date_to AND c.id in %s
        """
        cr.execute(sql,(tuple(ids),))
        return len(cr.fetchall())==0

    _constraints = [
        (_constraint_no_seances_closed_outher_dates, 'There are a seance closed other dates', ['date_from']),
    ]
    
        
    def unlink(self, cr, uid, ids, context=None):
        if self.pool.get('school.seance').search(cr, uid, [('classe_id','in',ids),('state','in',['closed'])]):
            raise orm.except_orm('Not programmed!','There are closes seances to remove. Delete not granted.')
        ret=super(school_classe, self).unlink(cr, uid, ids, context=context)
        return ret

    def write(self, cr, uid, ids, vals, context=None):
        if 'holidays_calendar_id' in vals:
            for classe in self.browse(cr, uid, ids):
                if classe.holidays_calendar_id:
                    for hi in classe.holidays_calendar_id.holidays:
                        self.pool.get('school.holiday_interval').modifica_calendari(cr, uid, [classe.id], hi.date_from, hi.date_to, None, None, context=context)
        ret=super(school_classe, self).write(cr, uid, ids, vals, context=context)
        if 'holidays_calendar_id' in vals:
            for classe in self.browse(cr, uid, ids):
                if classe.holidays_calendar_id:
                    for hi in classe.holidays_calendar_id.holidays:
                        self.pool.get('school.holiday_interval').modifica_calendari(cr, uid, [classe.id], None, None, hi.date_from, hi.date_to, context=context)
        if 'group_id' in vals:
            seance_close_ids=self.pool.get('school.seance').search(cr, uid, [('classe_id','in',ids),('state','=','closed')])
            if seance_close_ids:
                raise orm.except_orm('Error!','Any of the seances is closed. Group change not allowed.')

        return ret

    def create(self, cr, uid, vals, context=None):
        if 'name' not in vals or not vals['name']:
            course_id=vals['course_id']
            course_name=self.pool.get('school.course').browse(cr, uid, course_id).name
            group_id=vals.get('group_id',False)
            group_name=group_id and self.pool.get('groups.group').browse(cr, uid, group_id).name
            vals['name']="%s - %s" % (course_name,group_name)
        return super(school_classe, self).create(cr, uid, vals, context=context)
school_classe()

class school_seance(osv.osv):
    _name = 'school.seance'
school_seance()

class school_teacher_seance(osv.osv):
    _name = 'school.teacher_seance'
    _columns = {
        'teacher_id' : fields.many2one('school.teacher', 'Teacher', required=True, select=1, ),
        'seance_id' : fields.many2one('school.seance','Seance', required=True, select=1, ondelete='cascade' ),
        'title' : fields.selection([('imperative','Imperative'),('needed','Needed'),('optional','Optional'),('advisable','Advisable')], 'Title', ),
    }

    _sql_constraints = [('teacher_seance_unique','UNIQUE (teacher_id,seance_id)','A teacher entry must be unique!'),]

    def name_get(self, cr, uid, ids, context=None):
        pre_ret=[]
        seance_name_by_id={}
        teacher_name_by_id={}
        for item in self.browse(cr, uid, ids, context=context):
            seance_name_by_id[item.seance_id.id]=""
            teacher_name_by_id[item.teacher_id.id]=""
            pre_ret.append( (item.id,item.seance_id.id,item.teacher_id.id) )
        for (id,name) in self.pool.get('school.seance').name_get(cr, uid, seance_name_by_id.keys(), context=context):
            seance_name_by_id[id]=name
        for (id,name) in self.pool.get('school.teacher').name_get(cr, uid, teacher_name_by_id.keys(), context=context):
            teacher_name_by_id[id]=name
        return [(id,"%s - %s" % (teacher_name_by_id[teacher_id],seance_name_by_id[seance_id])) for (id,seance_id,teacher_id) in pre_ret]
        
school_teacher_seance()


class school_seance(osv.osv):
    _name = 'school.seance'

    def _get_group(self, cr, uid, ids, field_name, arg, context=None):
        res={}
        for item in self.browse(cr, uid, ids):
            group_id=None
            if not group_id:
                group_id=item.classe_id.group_id
            res[item.id]=group_id.id
        return res

    def _get_partinotes(self, cr, uid, ids, field_name, arg, context):
        """
        Funció que retorna els contactes que participen en una sessió i les
        anotacions que tenen els contactes que tinguin temps comú amb la sessió.
        """
        res={}
        data=self.browse(cr, uid, ids)
        for item in data:
            if not item.group_id:
                # Si la sessió no té grup llavors no té participacions
                res[item.id]={'contact_ids': [], 'annotation_ids': [],}
                continue
            # preparem algunes variables pel càlcul
            datetime_from = item.date
            datetime_to = item.date_to
            group_id=item.group_id.id
            priority_group=item.group_id.priority
            # omplim gas amb els id de les assignacions al grup que tinguin temps dins de la sessió
            gas=self.pool.get('groups.group_assignation').search(cr, uid, [('group_id','=',group_id),('limit_from','<',datetime_to),('limit_to','>',datetime_from)], order='limit_from')
            # amb les id de les assignacions llegim els identificadors de les participacions i les dispositem com a claus amb un diccionari amb valors els intervals de temps on tenen assignacions en el grup
            part_data={}
            for data2 in self.pool.get('groups.group_assignation').read(cr, uid, gas, ['participation_id','limit_from','limit_to']):
                if data2['participation_id'] not in part_data: part_data[data2['participation_id']]=[]
                part_data[data2['participation_id']].append( ( max(datetime_from,data2['limit_from'],), min(datetime_to,data2['limit_to'],) ) )
            # demanem al mètode auxiliar els grups que tenen assignats participacions dins de l'interval de la sessió.
            part_groups=self.pool.get('groups.participation').groups_in_interval(cr, uid, part_data.keys(), datetime_from, datetime_to, context=context)
            # en aquest punt podriem agafar només el grup prioritari de cada participació i si és el mateix que el grup de la sessio confirmem que hi participa. Però, i si hi ha un grup més prioritari? Per treure-li la participació a aquesta sessió, el grup prioritari ha de tenir sessió en el mateix moment, no? Haurem de preguntar-ho.
            group_ids=set([0])
            for value in part_groups.values(): group_ids.update([x['group_id'] for x in value])
            query="SELECT group_id,date,date_to FROM school_seance WHERE state NOT IN ('wrong','suspended') AND id<>%s AND group_id IN %s AND date<%s and date_to>%s ORDER BY date"
            cr.execute(query,(item['id'],tuple(group_ids,),datetime_to,datetime_from))
            seances_by_groups={}
            for (group_id,date,date_to) in cr.fetchall():
                if group_id not in seances_by_groups: seances_by_groups[group_id]=[]
                seances_by_groups[group_id]+=[{'date': date, 'date_to': date_to}]
            # guardem les dades de les sessions per grups amb assignacions de les participacions del grup de la sessió que volem trobar.

            part_ids_real=[] # participacions reals tenint en compte les sessions per grups prioritaris
            for (part_id,intervals_yes) in part_data.items():
                # Per cada participació començarem amb els intervals on sí assisteix a la sessió per tenir assignacions. Normalment només hi haurà una assignació. Però no descartem la possibilitat que un o varis cops hi hagi canvis de grup (varies assignacions a aquest grup) durant el transcurs de la sessió.
                for part_group in part_groups.get(part_id, []):
                    # recorrerem per ordre els grups de la participació
                    if part_group['priority']<=priority_group: continue
                    # si es un menys prioritari, no fem cas i passem a un altre grup. Com que està ordenat la resta d'elements no es tractaran.
                    for seance_group in seances_by_groups.get(part_group['group_id'],[]):
                        # passarem per totes les sessions d'aquest grup prioritari de la participació
                        # preparem l'interval de la sessio
                        datetime_from2 = seance_group['date']
                        datetime_to2 = seance_group['date_to']
                        new_intervals_yes=[] # llista on anirem posant els intervals que es salven de la cerca en aquesta sessio
                        # mirarem cadascun dels intervals en positiu que tenim
                        for (i_from,i_to) in intervals_yes:
                            if datetime_from2>i_from:
                                new_intervals_yes.append((i_from,min(datetime_from2,i_to,),))
                            if datetime_to2<i_to:
                                new_intervals_yes.append((max(datetime_to2,i_from,),i_to,))
                        # actualitzarem els intervals en la llista de referencia
                        intervals_yes=new_intervals_yes
                if intervals_yes: # si queden interval anotarem com que assisteix. A veure... podria ser que només assistis 5 minuts. A estudiar com informar perquè en aquest diccionari els intervals de la participació.
                    part_ids_real.append(part_id)
                            
            contact_ids=list(set([x['participant'][0] for x in self.pool.get('groups.participation').read(cr, uid, part_ids_real, ['participant'])]))
            res[item['id']]={'contact_ids': contact_ids,}
            res[item['id']]['annotation_ids']=self.pool.get('contact.annotation').search(cr, uid, [('contact_id','in',contact_ids),('valid_from','<',datetime_to),('valid_to','>',datetime_from)], context=context)
        return res

    def name_get(self, cr, uid, ids, context=None):
        ret=[]
        for seance in self.browse(cr, uid, ids):
            ret.append((seance.id,"%(classe_name)s-%(date)s-%(room_name)s" % {
                'classe_name':seance.classe_id.name,
                'date': seance.date,
                'room_name': seance.room_id.name,
                }))
        return ret

    def _main_teacher(self, cr, uid, ids, field_name, arg, context=None):
        ret={}
        priority=dict([('imperative',0),('needed',1),('optional',2),('advisable',3)])
        for seance in self.browse(cr, uid, ids, context):
            ret[seance.id]=None
            priority_act=-1
            for teacher_data in seance.teachers:
                if priority_act<0 or priority[teacher_data.title]<priority_act:
                    ret[seance.id]=teacher_data.teacher_id.id
        return ret

    def _seance_for_contact_search(self, cr, uid, obj, name, args, context=None):
        #TODO: Todo.
        return []

    def _seance_for_teacher_search(self, cr, uid, obj, name, args, context=None):
        args2=[]
        teacher_args=[]
        for (field,op,value) in args:
            trossos=field.split('.', 1)
            if trossos[0]=='one_teacher':
                if len(trossos)==1:
                    teacher_args+=[('name',op,value)]
                else:
                    teacher_args+=[(trossos[1],op,value)]
            else:
                args2+=[(field,op,value)]
        if teacher_args:
            teacher_ids=self.pool.get('school.teacher').search(cr, uid, teacher_args)
            if teacher_ids:
                query="SELECT seance_id FROM school_teacher_seance WHERE teacher_id IN %s"
                cr.execute(query, (tuple(teacher_ids),))
                args2+=[('id','in',[seance_id for (seance_id) in cr.fetchall()])]
            else:
                args2+=[('id','=',False)]
        return args2

    _columns = {
        'room_id' : fields.many2one('school.room','Room', required=False, select=1, help="The seance's room"),
        'teacher_id': fields.function(_main_teacher,type='many2one',obj='school.teacher',method=True,string="Main teacher",),
        'one_teacher' : fields.function(lambda self,cr,uid,ids,*a: dict.fromkeys(ids,False),type='many2one',obj='school.teacher',method=True,string="Teacher",fnct_search=_seance_for_teacher_search,select=1,help='For search seances by teacher'),
        'teachers' : fields.one2many('school.teacher_seance','seance_id',),
        'date' : fields.datetime('Date', required=True, select=1),
        'date_to' : fields.datetime('Date to', required=True, select=1,),
        'classe_id' : fields.many2one('school.classe', 'Class', required=True, help="The class", ondelete="cascade", readonly=True, select=1),
        'course_id': fields.related('classe_id','course_id',type='many2one',related='school.course'),
        'group_id' : fields.function(_get_group, method=True, string="Group",
            type="many2one", obj='groups.group', select=1, store={
                'school.classe': ( lambda self, cr, uid, ids, context: self.pool.get('school.seance').search(cr, uid, [('classe_id','in',ids)]), ['group_id'], 10),
                },
            ),
        'contact_ids' : fields.function(_get_partinotes, type="one2many", obj='res.partner.contact', fnct_search=_seance_for_contact_search, method=True,multi='partinotes'),
        'annotation_ids' : fields.function( _get_partinotes, type='one2many', obj='contact.annotation', method=True,multi='partinotes'),
        'state' : fields.selection([('created', 'Created'),('ready','Ready'),('suspended','Suspended'),('wrong','Wrong'),
                                    ('opened', 'Editing'),
                                    ('closed', 'Closed')],
                                   'State',
                                   readonly=False,
                                   help="The state of the seance",select=True
                                  ),
        'change_state_log' : fields.text('Change State Log'),
    }

    _order = 'date'

    _defaults = {
        'state' : lambda *a: 'created',         
    }

    def _check_participation_in_seance(self, cr, uid, ids, context=None):
        """
        No group of that seances could have a participations who have
        assignations (in the same time interval) in other groups, with
        the same priority, which they will have seances at the same time
        interval
        """
        query="""
        SELECT ga.id FROM
            groups_group_assignation AS ga,
            groups_group AS g,
            school_seance AS s,
            groups_group_assignation AS ga2,
            groups_group AS g2,
            school_seance AS s2
        WHERE
            ga.group_id=g.id AND
            ga.participation_id=ga2.participation_id AND
            ga2.group_id=g2.id AND
            g2.id<>g.id AND
            g.priority=g2.priority AND
            s.group_id=g.id AND
            s.state NOT IN ('created','suspended','wrong') AND
            s2.group_id=g2.id AND
            s2.state NOT IN ('created','suspended','wrong') AND
            s2.id<>s.id AND
CASE WHEN s.date_to < ga.limit_to
    THEN s.date_to ELSE ga.limit_to END
    >
CASE WHEN s2.date > ga2.limit_from THEN s2.date ELSE ga2.limit_from END
            AND
CASE WHEN s.date > ga.limit_from THEN s.date ELSE ga.limit_from END
    <
CASE WHEN s2.date_to < ga2.limit_to
    THEN s2.date_to ELSE ga2.limit_to END
        AND s.id IN %s LIMIT 1
        """
        cr.execute(query, (tuple(ids),) )
        if cr.fetchall():
            return False
        return True

    def _check_groups_in_seance(self, cr, uid, ids, context=None):
        query="""
        SELECT DISTINCT g.id,g.name,s.classe_id,s2.classe_id FROM
            groups_group AS g,
            school_seance AS s,
            school_seance AS s2
        WHERE
            s.group_id=g.id AND
            s2.group_id=g.id AND
            s2.id<>s.id AND
            s.state NOT IN ('created','suspended','wrong') AND
            s2.state NOT IN ('created','suspended','wrong') AND
s.date_to > s2.date AND s.date < s2.date_to
        AND s.id IN %s LIMIT 1
        """
        cr.execute(query, (tuple(ids),) )
        if cr.fetchall():
            return False
        return True

    _constraints = [
        (_check_groups_in_seance, 'Another seance approved for the group at same time!',['date']),
        (_check_participation_in_seance, 'One or more participation are included in other seance approved at same priority',['date']),
        ]

    def compute_wrong_seances(self, cr, uid, limit=100, context=None):
            wrong_classes=set()
            query="""
            SELECT DISTINCT s.date,g.id,g.name,s.id,s2.id,s.classe_id FROM
                school_seance AS s INNER JOIN
                groups_group AS g ON s.group_id=g.id LEFT JOIN
                school_seance AS s2 ON (
                    s2.group_id=g.id AND
                    s2.id<>s.id AND
                    s2.state NOT IN ('suspended','wrong') AND
                    s.date_to > s2.date AND
                    s.date < s2.date_to
                )
            WHERE s.state='created'
            ORDER BY s.date
            LIMIT %s
            """
            cr.execute(query, (limit,))
            ids_to_ready=set()
            ids_to_wrong={}
            for (sdate,gid,gname,sid,s2id,sclasse_id,) in cr.fetchall():
                if s2id:
                    if sid not in ids_to_wrong: ids_to_wrong[sid]=''
                    ids_to_wrong[sid]+='The same group has two seances simultaneously. This one and other with id %s.\n' % s2id
                    wrong_classes.add(sclasse_id)
                else:
                    ids_to_ready.add(sid)
            print "PAS 1: Total %s, Erronis %s" % (len(ids_to_wrong)+len(ids_to_ready),len(ids_to_wrong))
            if ids_to_ready:
                """
                No group of that seances could have a participations who have
                assignations (in the same time interval) in other groups, with
                the same priority, which they will have seances at the same time
                interval
                """
                query="""
                SELECT s.id,ga.participation_id,s2.id,s.classe_id FROM
                    groups_group_assignation AS ga,
                    groups_group AS g,
                    school_seance AS s,
                    groups_group_assignation AS ga2,
                    groups_group AS g2,
                    school_seance AS s2
                WHERE
                    ga.group_id=g.id AND
                    ga.participation_id=ga2.participation_id AND
                    ga2.group_id=g2.id AND
                    g2.id<>g.id AND
                    g.priority=g2.priority AND
                    s.group_id=g.id AND
                    s2.group_id=g2.id AND
                    s2.state NOT IN ('suspended','wrong') AND
                    s2.id<>s.id AND
        CASE WHEN s.date_to < ga.limit_to
            THEN s.date_to ELSE ga.limit_to END
            >
        CASE WHEN s2.date > ga2.limit_from THEN s2.date ELSE ga2.limit_from END
                    AND
        CASE WHEN s.date > ga.limit_from THEN s.date ELSE ga.limit_from END
            <
        CASE WHEN s2.date_to < ga2.limit_to
            THEN s2.date_to ELSE ga2.limit_to END
                AND s.id IN %s
                """
                cr.execute(query, (tuple(ids_to_ready),) )
                for (sid,gaid,s2id,sclasse_id,) in cr.fetchall():
                    if sid in ids_to_ready:
                        ids_to_ready.remove(sid)
                        if sid not in ids_to_wrong: ids_to_wrong[sid]=''
                        ids_to_wrong[sid]+='The same participation with id %s in the seance has two seances simultaneously. This one and other with id %s.\n' % (gaid,s2id)
                        wrong_classes.add(sclasse_id)
                print "PAS 2: Total %s, Erronis %s" % (len(ids_to_wrong)+len(ids_to_ready),len(ids_to_wrong))
            #TODO: 'classe_ids_check_ready': cids_to_ready,
            return {'seance_ids_check_ready': list(ids_to_ready), 'seance_ids_dict_with_errors': ids_to_wrong, 'classe_ids_check_wrong': list(wrong_classes), }

    def compute_created_seances(self, cr, uid, context=None):
        dicci=self.compute_wrong_seances(cr, uid, limit=100, context=None)
        self.write(cr, uid, dicci['seance_ids_check_ready'], {'state': 'ready'})
        for (key,log_text) in dicci['seance_ids_dict_with_errors'].items():
            self.write(cr, uid, [key], {'state': 'wrong','change_state_log': log_text})
        if dicci['classe_ids_check_wrong']: self.pool.get('school.classe').write(cr, uid, dicci['classe_ids_check_wrong'], {'wrong_seances': True})
        return True



    #
    #  Never unlink a closed seance
    #
    def unlink(self, cr, uid, ids, context=None):
        """Never unlink a closed seance"""
        for seance in self.read(cr, uid, ids, ['state'], context=context):
            if seance['state']=='closed':
                raise orm.except_orm('Not programmed!','One of the seances is closed. Delete not programmed.')
        ret=super(school_seance, self).unlink(cr, uid, ids, context=context)
        return ret

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        ret=super(school_seance, self).search(cr, uid, args, offset=offset, limit=limit, order=order, context=context, count=count)
        return ret

    def write(self, cr, uid, ids, vals, context=None):
        for data in self.read(cr, uid, ids, ['state']):
            if data['state']=='closed':
                raise orm.except_orm('Seance closed!','A change over closed seance detected.')
        if 'group_id' in vals or 'date' in vals or 'room_id' in vals or 'date_to' in vals:
            vals['state']='created'
        ret=super(school_seance, self).write(cr, uid, ids, vals, context=context)
        return ret
school_seance()

class school_substitution(osv.osv):
    _name = "school.substitution"
    
    def _teacher_out_seances(self, cr, uid, ids, context=None):
        ret={}
        for item in self.browse(cr, uid, ids):
            cr.execute("SELECT s.id FROM school_seance AS s INNER JOIN school_teacher_seance AS ts ON s.id=ts.seance_id AND ts.teacher_id=%s WHERE s.date>%s AND s.date_to <%s", (item.teacher_out.id, item.date_begin, item.date_end) )
            ret[item.id]=[x for (x,) in cr.fetchall()]
        return ret
    
    _columns = {
        'teacher_out': fields.many2one('school.teacher','Teacher out',required=True,),
        'teacher_in' : fields.many2one('school.teacher','Teacher in',required=True,),
        'date_begin' : fields.datetime('Begin',help='Date time to begin substitution',required=True,),
        'date_end' : fields.datetime('End',help='Date time to end substitution',required=True,),
    }
    
    def create(self, cr, uid, vals, context=None):
        ret=super(school_substitution, self).create(cr, uid, vals, context=context)
        if ret:
            seance_ids=self._teacher_out_seances(cr, uid, [ret], context)
            teacher_seance_ids=self.pool.get('school.teacher_seance').search(cr, uid, [('seance_id','in',seance_ids),('teacher_out','=',vals['teacher_out'])])
            self.pool.get('school.teacher_seance').write(cr, uid, teacher_seance_ids, {'teacher_id': vals['teacher_in']})
        return ret
    
school_substitution()

class school_holiday_interval(osv.osv):
    _name = 'school.holiday_interval'
    _inherit = 'school.holiday_interval'

    def modifica_calendari(self, cr, uid, classe_ids, date_from_ant, date_to_ant, date_from_pos, date_to_pos, context=None):
        if date_from_pos and date_to_pos:
            args=[('classe_id','in',classe_ids)]
            args+=[('date','>=',date_from_pos),('date','<=',date_to_pos)]
            if date_from_ant and date_to_ant:
                args+=['|',('date','<',date_from_ant),('date','>',date_to_ant)]
            seances_to_unlink=self.pool.get('school.seance').search(cr, uid, args)
            self.pool.get('school.seance').unlink(cr, uid, seances_to_unlink)

    def write(self, cr, uid, ids, vals, context=None):
        if 'date_from' in vals or 'date_to' in vals:
            for hi in self.browse(cr, uid, ids):
                (date_from_ant,date_to_ant)=(hi.date_from,hi.date_to)
                (date_from_pos,date_to_pos)=(vals.get('date_from',date_from_ant),vals.get('date_to',date_to_ant))
                if (date_from_ant,date_to_ant)==(date_from_pos,date_to_pos): continue
                classe_ids=[]
                for cal in hi.calendars:
                    for classe in cal.classe_ids:
                        classe_ids.append(classe.id)
                self.modifica_calendari(cr, uid, classe_ids, date_from_ant, date_to_ant, date_from_pos, date_to_pos, context=context)
        return super(school_holiday_interval, self).write(cr, uid, ids, vals, context=context)

    def unlink(self, cr, uid, ids, context=None):
        for hi in self.browse(cr, uid, ids):
            (date_from_ant,date_to_ant)=(hi.date_from,hi.date_to)
            classe_ids=[]
            for cal in hi.calendars:
                for classe in cal.classe_ids:
                    classe_ids.append(classe.id)
            self.modifica_calendari(cr, uid, classe_ids, date_from_ant, date_to_ant, None, None, context=context)
        ret=super(school_holiday_interval, self).unlink(cr, uid, ids, context=context)
        return ret

    def create(self, cr, uid, vals, context=None):
        ret=super(school_holiday_interval, self).create(cr, uid, vals, context=context)
        hi=self.browse(cr, uid, ret)
        classe_ids=[]
        for cal in hi.calendars:
            for classe in cal.classe_ids:
                classe_ids.append(classe.id)
        self.modifica_calendari(cr, uid, classe_ids, None, None, hi.date_from, hi.date_to, context=context)
        return ret

school_holiday_interval()

class school_holidays_calendar(osv.osv):
    _name = 'school.holidays_calendar'
    _inherit = 'school.holidays_calendar'
    _columns = {
        'classe_ids' : fields.one2many('school.classe','holidays_calendar_id',string="Classes"),
    }

    def write(self, cr, uid, ids, vals, context=None):
        hi_obj=self.pool.get('school.holiday_interval')
        for data in self.read(cr, uid, ids, ['holidays','classe_ids']):
            classe_ids=data['classe_ids']
            holidays=data['holidays']
            classe_ids_pos=classe_ids
            holidays_pos=holidays
            # Tornar a crear sessions de les classes que treiem
            classes_surten=[]
            classes_entren=[]
            holidays_surten=[]
            holidays_entren=[]
            if 'classe_ids' in vals:
                classe_ids_pos=vals['classe_ids'][0][2]
                classes_surten=list(set(classe_ids).difference(classe_ids_pos))
                classes_entren=list(set(classe_ids_pos).difference(classe_ids))
            classes_mantenen=list(set(classe_ids).difference(classes_surten))
            if 'holidays' in vals:
                holidays_pos=vals['holidays'][0][2]
                holidays_surten=list(set(holidays).difference(holidays_pos))
                holidays_entren=list(set(holidays_pos).difference(holidays))
            for hi in hi_obj.browse(cr, uid, holidays):
                hi_obj.modifica_calendari(cr, uid, classes_surten, hi.date_from, hi.date_to, None, None, context=context)
            for hi in self.pool.get('school.holiday_interval').browse(cr, uid, holidays_surten):
                hi_obj.modifica_calendari(cr, uid, classes_mantenen, hi.date_from, hi.date_to, None, None, context=context)
            for hi in self.pool.get('school.holiday_interval').browse(cr, uid, holidays_entren):
                hi_obj.modifica_calendari(cr, uid, classes_mantenen, None, None, hi.date_from, hi.date_to, context=context)
            for hi in self.pool.get('school.holiday_interval').browse(cr, uid, holidays_pos):
                hi_obj.modifica_calendari(cr, uid, classes_entren, None, None, hi.date_from, hi.date_to, context=context)
        print "school_holidays_calendar: write: cr, uid, ids, vals, context: ", cr, uid, ids, vals, context
        ret=super(school_holidays_calendar, self).write(cr, uid, ids, vals, context=context)
        return ret

    def unlink(self, cr, uid, ids, context=None):
        print "school_holidays_calendar: unlink: cr, uid, ids, context: ", cr, uid, ids, context
        self.write(cr, uid, ids, {'classe_ids': [(6,0,[])]})
        ret=super(school_holidays_calendar, self).unlink(cr, uid, ids, context=context)
        return ret

    def create(self, cr, uid, vals, context=None):
        print "school_holidays_calendar: create: cr, uid, vals, context: ", cr, uid, vals, context
        ret=super(school_holidays_calendar, self).create(cr, uid, {}, context=context)
        self.write(cr, uid, [ret], vals)
        return ret

school_holidays_calendar()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
