# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#      Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#    Copyright (C) 2011 Zikzakmedia S.L. (http://www.zikzakmedia.com)
#      Jesús Martín Jiménez <jmargin@zikzakmedia.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program. If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
from timed_groups.timed_groups import _MIN_DATETIME,_MAX_DATETIME

class groups_group(osv.osv):
    _name = 'groups.group'
    _inherit = 'groups.group'

    _columns = {
        'seance_ids' : fields.one2many('school.seance', 'group_id', string='Seances',),
    }
        
groups_group()

class groups_group_assignation(osv.osv):
    _name = 'groups.group_assignation'
    _inherit = 'groups.group_assignation'

    def _wrong_seances(self, cr, uid, ids, field_name, arg, context = None):
        ret = {}
        for ga in self.browse(cr, uid, ids):
            ret[ga.id] = {'wrong': False, 'coincidences': [], 'wrong_seances': []}
            query = """
    SELECT d.id,s.id FROM school_seance AS s INNER JOIN (

    SELECT DISTINCT ga2.id,ga2.group_id,greatest(s.date,ga.datetime_from) AS datetime_from,least(s.date_to,ga.datetime_to) AS datetime_to FROM
            groups_group_assignation AS ga
            INNER JOIN groups_group AS g ON ga.group_id = g.id
            INNER JOIN groups_group_assignation AS ga2 ON ga.participation_id = ga2.participation_id AND ga.group_id <> ga2.group_id
            INNER JOIN groups_group AS g2 ON ga2.group_id = g2.id AND COALESCE(g.priority,0) = COALESCE(g2.priority,0)
            INNER JOIN school_seance AS s ON ga.group_id = s.group_id AND ga.datetime_from < s.date_to AND ga.datetime_to > s.date
            WHERE ga.id = %s

    ) AS d ON s.group_id = d.group_id AND s.date < d.datetime_to AND s.date_to > d.datetime_from
            """
            cr.execute(query, (ga.id,) )
            for (ga2id,sid) in cr.fetchall():
                ret[ga.id]['wrong'] = True
                if ga2id not in ret[ga.id]['coincidences']: ret[ga.id]['coincidences'].append(ga2id)
                ret[ga.id]['wrong_seances'].append(sid)
        return ret
        
    _columns =  {
        'wrong' : fields.function(_wrong_seances, type = 'boolean', method = True, string = 'Wrong', multi = 'seances'),
        'coincidences' : fields.function(_wrong_seances, type = 'many2many', obj = 'groups.group_assignation', method = True, string = 'Concidences', multi = 'seances'),
        'wrong_seances' : fields.function(_wrong_seances, type = 'many2many', obj = 'school.seance', method = True, string = "Wrong seances", multi = "seances",),
    }

groups_group_assignation()

class groups_participation(osv.osv):
    _name = 'groups.participation'
    _inherit = 'groups.participation'

    def get_seances(self, cr, uid, ids, datetime_from = _MIN_DATETIME, datetime_to = _MAX_DATETIME, context = None):
        dicci = {}
        for id in ids: dicci[id] = []
        query = """
SELECT ga.participation_id,s.id as seance_id,greatest(s.date,%(datetime_from)s) as datetime_from, least(s.date_to,%(datetime_to)s) as datetime_to, g.priority
FROM groups_group_assignation AS ga
INNER JOIN groups_group AS g ON ga.group_id=g.id
INNER JOIN school_seance AS s ON s.state NOT IN ('suspended') AND ga.group_id=s.group_id AND (s.date, s.date_to) OVERLAPS (ga.datetime_from,ga.datetime_to)
WHERE ga.participation_id IN %(ids)s AND s.date_to>%(datetime_from)s AND s.date<%(datetime_to)s
GROUP BY ga.participation_id,s.id,s.date,s.date_to,g.priority
ORDER BY greatest(s.date,%(datetime_from)s),g.priority"""
        cr.execute(query, {'ids': tuple(ids), 'datetime_from': datetime_from, 'datetime_to': datetime_to})
        for (part_id,seance_id,dt_from,dt_to,priority) in cr.fetchall():
            if not dicci[part_id]: dicci[part_id] = []
            inclos = False
            nova_llista = []
            for (sid,dtf,dtt,pri) in dicci[part_id]:
                if dt_from<dtt and dt_to>dtf and priority!=pri:
                    inclos = True
                    if priority > pri:
                        if dtf < dt_from:
                            nova_llista.append( (sid,dtf,dt_from,pri) )
                        nova_llista.append( (seance_id,dt_from,dt_to,priority) )
                        if dtt > dt_to:
                            nova_llista.append( (sid,dt_to,dtt,pri) )
                    elif pri < priority:
                        if dt_from < dtf:
                            nova_llista.append( (seance_id,dt_from,dtf,priority) )
                        nova_llista.append( (sid,dtf,dtt,pri) )
                        if dtt > dt_to:
                            nova_llista.append( (seance_id,dtt,dt_to,priority) )
                else:
                    if dtf >= dt_from and not inclos:
                        nova_llista.append( (seance_id,dt_from,dt_to,priority) )
                        inclos = True
                    nova_llista.append( (sid,dtf,dtt,pri) )
            if not inclos: nova_llista.append( (seance_id,dt_from,dt_to,priority) )
            dicci[part_id] = nova_llista
        return dicci

groups_participation()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
