import copy
# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from lxml import etree
from osv import orm, osv, fields
import base64
from math import floor

def floats_to_school_fet_hour(hour_from,hour_to):
    return "%02d:%02d-%02d:%02d" % (hour_from,(hour_from-floor(hour_from))*60,hour_to,(hour_to-floor(hour_to))*60) 
    

class school_fet_hour(osv.osv):
    _name = 'school.fet.hour'

    def name_get(self, cr, uid, ids):
        ret=[]
        for item in self.browse(cr, uid, ids):
            ret.append( (item.id, floats_to_school_fet_hour(item.hour_from,item.hour_to) ) )
        return ret

    _columns = {
        'sequence' : fields.integer('Sequence',select=1,),
        'hour_from' : fields.float('Hour from',),
        'hour_to' : fields.float('Hour to',),
    }

    def _constraint_no_overlap_hours(self, cr, uid, ids):
        sql="""
        SELECT fh1.id
        FROM school_fet_hour AS fh1 INNER JOIN school_fet_hour AS fh2 ON fh1.id<>fh2.id AND fh1.hour_to>fh2.hour_from AND fh1.hour_from<fh2.hour_to
        WHERE fh1.id IN %s
        """
        cr.execute(sql,(tuple(ids),))
        return len(cr.fetchall())==0

    _constraints = [
        (_constraint_no_overlap_hours, 'Overlaping', ['hour_from','hour_to']),
    ]
    _sql_constraints = [
        ('hour_from_ok','CHECK (hour_from BETWEEN 0 AND 24)','Hour incorrect'),
        ('hour_to_ok','CHECK (hour_to BETWEEN 0 AND 24)','Hour incorrect'),
        ('hours_ok','CHECK (hour_to>hour_from)','Hours incorrect'),
    ]

school_fet_hour()

class school_fet_day(osv.osv):
    _name = 'school.fet.day'

    _columns = {
        'name' : fields.selection([('monday','monday'),('tuesday','tuesday'),('wednesday','wednesday'),('thursday','thursday'),('friday','friday'),('saturday','saturday'),('sunday','sunday')],"Week day",),
    }

    _sql_constraints = [
        ('name_uniq', 'unique (name)', 'The day name must be unique !')
    ]
school_fet_day()

class school_fet_aules_propies(osv.osv):
    _name = 'school.fet.aules_propies'

    _columns = {
        'group_id' : fields.many2one('groups.group',string="Group",ondelete="cascade",),
        'room_id' : fields.many2one('school.room',string="Room",ondelete="cascade",),
        'percentatge' : fields.integer('Percentatge',),
    }
school_fet_aules_propies()

class school_fet_export(osv.osv_memory):
    _name = 'school.fet.export'
    
    _columns = {
        'iwl_ids' : fields.many2many('school.impartition_week_line','school_fet_export_iwl_rel','fetex_id','iwl_id',string='IWLs',),
        'file' : fields.binary('File',),
    }

    _defaults = {
        'iwl_ids' : lambda self,cr,uid,context={} : context.get('active_ids',[])
    }

    def act_cancel(self, cr, uid, ids, context=None):
        pass
    
    def generate_file(self, cr, uid, ids, context=None):
        root=etree.Element('fet')
        root.set("version","5.13.4")
        iname=etree.SubElement(root,"Institution_Name")
        iname.text="A Institution Name"
        iname=etree.SubElement(root,"Comments")
        iname.text="Comentaris predeterminats"

        # Llista d'hores
        hlist=etree.SubElement(root,"Hours_List")
        fet_hour_ids=self.pool.get('school.fet.hour').search(cr, uid, [], context=context)
        if not fet_hour_ids:
            raise orm.except_orm('No Hours!','Impossible FET exportation. We need hours!')
        nhlist=etree.SubElement(hlist,"Number")
        nhlist.text=str(len(fet_hour_ids))
        for (hour_id,hour_name) in self.pool.get('school.fet.hour').name_get(cr, uid, fet_hour_ids):
            elem=etree.SubElement(hlist,"Name")
            elem.text=hour_name

        # Llista de dies
        dlist=etree.SubElement(root,"Days_List")
        fet_day_ids=self.pool.get('school.fet.day').search(cr, uid, [], context=context)
        if not fet_day_ids:
            raise orm.except_orm('No Days!','Impossible FET exportation. We need days!')
        ndlist=etree.SubElement(dlist,"Number")
        ndlist.text=str(len(fet_day_ids))
        for data in self.pool.get('school.fet.day').read(cr, uid, fet_day_ids, ['name'], context=context):
            elem=etree.SubElement(dlist,"Name")
            elem.text=data['name']

        # Llista d'edificis
        building_list=etree.Element("Buildings_List")
        ids2=self.pool.get('school.building').search(cr, uid, [], context=context)
        for item in self.pool.get('school.building').browse(cr, uid, ids2, context=context):
            subelement=etree.SubElement(building_list,"Building")
            subelement=etree.SubElement(subelement,"Name")
            subelement.text="%s,%s" % (item.id,item.name)

        # Llista d'aules
        room_list=etree.Element("Rooms_List")
        rooms=set()
        ids2=self.pool.get('school.room').search(cr, uid, [], context=context)
        for item in self.pool.get('school.room').browse(cr, uid, ids2, context=context):
            room_key="%s,%s" % (item.id,item.name)
            rooms.add(room_key)
            subelement=etree.SubElement(room_list,"Room")
            subelement2=etree.SubElement(subelement,"Name")
            subelement2.text=room_key
            if item.building_id:
                subelement2=etree.SubElement(subelement,"Building")
                subelement2.text="%s,%s" % (item.building_id.id,item.building_id.name)
            subelement2=etree.SubElement(subelement,"Capacity")
            subelement2.text=str(item.capacity)

        for fex in self.browse(cr, uid, ids, context=context):
            root2=copy.deepcopy(root)
            # contemplem només l'assignació de grups en aquest moment
            parts=set()
            groups={}
            teachers=set()
            courses=set()
            activities={}
            iwl_ids=[]
            teacher_ids=set()
            for iwl in fex.iwl_ids:
                iwl_ids.append(iwl.id)
                activity={'teachers': set(),'classe_id': iwl.classe_id.id,}
                activity_key=str(iwl.id)
                activities[activity_key]=activity
                course_key="%s,%s" % (iwl.classe_id.course_id.id,iwl.classe_id.course_id.name)
                courses.add(course_key)
                for teacher_data in iwl.teachers:
                    teacher_key="%s,%s" % (teacher_data.teacher_id.id,teacher_data.teacher_id.teacher_name)
                    teacher_ids.add(teacher_data.teacher_id.id)
                    teachers.add(teacher_key)
                    activity['teachers'].add(teacher_key)
                activity['course']=course_key

                group_key="%s,%s" % (iwl.group_id.id,iwl.group_id.name)
                activity['group']=group_key
                if group_key not in groups:
                    groups[group_key]=set()
                    query="SELECT ga.participation_id FROM groups_group_assignation AS ga WHERE ga.group_id=%s AND now() BETWEEN ga.datetime_from AND ga.datetime_to"
                    cr.execute(query,(iwl.group_id.id,))
                    for (part_id,) in cr.fetchall():
                        parts.add(part_id)
                        groups[group_key].add(part_id)


            # Llista d'estudiants
            slist=etree.SubElement(root2,"Students_List")
            year=etree.SubElement(slist,"Year")
            year_name=etree.SubElement(year,"Name")
            year_name.text="now"
            year_nos=etree.SubElement(year,"Number_of_Students")
            year_nos.text=str(len(parts))
            for (group_key,student_set) in groups.items():
                group=etree.SubElement(year,"Group")
                group_name=etree.SubElement(group,"Name")
                group_name.text=group_key
                group_nos=etree.SubElement(group,"Number_of_Students")
                group_nos.text=str(len(student_set))
                for student in student_set:
                    subgroup=etree.SubElement(group,"Subgroup")
                    subgroup_name=etree.SubElement(subgroup,"Name")
                    subgroup_name.text=str(student)
                    subgroup_nos=etree.SubElement(subgroup,"Number_of_Students")
                    subgroup_nos.text="1"
            
            tlist=etree.SubElement(root2,"Teachers_List")
            for teacher_key in teachers:
                subelement=etree.SubElement(tlist,"Teacher")
                subelement2=etree.SubElement(subelement,"Name")
                subelement2.text=teacher_key
            su_list=etree.SubElement(root2,"Subjects_List")
            for course_key in courses:
                subelement=etree.SubElement(su_list,"Subject")
                subelement2=etree.SubElement(subelement,"Name")
                subelement2.text=course_key
            at_list=etree.SubElement(root2,"Activity_Tags_List")
            a_list=etree.SubElement(root2,"Activities_List")
            for (activity_key,activity) in activities.items():
                subelement=etree.SubElement(a_list,"Activity")
                for teacher_key in activity['teachers']:
                    subelement2=etree.SubElement(subelement,"Teacher")
                    subelement2.text=teacher_key
                subelement2=etree.SubElement(subelement,"Subject")
                subelement2.text=activity['course']
                subelement2=etree.SubElement(subelement,"Duration")
                subelement2.text="1"
                subelement2=etree.SubElement(subelement,"Id")
                subelement2.text=activity_key
                subelement2=etree.SubElement(subelement,"Activity_Group_Id")
                subelement2.text=str(activity['classe_id'])
                subelement2=etree.SubElement(subelement,"Active")
                subelement2.text="true"
                subelement2=etree.SubElement(subelement,"Students")
                subelement2.text=activity['group']
            
            root2.append( copy.deepcopy(building_list) )
            root2.append( copy.deepcopy(room_list) )
            
            tc_list=etree.SubElement(root2,"Time_Constraints_List")
            cbct=etree.SubElement(tc_list,"ConstraintBasicCompulsoryTime")
            wp=etree.SubElement(cbct,"Weight_Percentage")
            wp.text="100"

            # Not available teacher hours
            if teacher_ids:
                query="""
                SELECT td.teacher_id,iwl.week_day,sfh.hour_from,sfh.hour_to
                FROM school_impartition_week_line AS iwl
                    INNER JOIN school_teacher_data AS td ON iwl.id=td.iwl_id
                    INNER JOIN school_fet_hour AS sfh ON iwl.hour_from<sfh.hour_to AND iwl.hour_from+iwl.duration>sfh.hour_from
                WHERE iwl.week_day IS NOT NULL AND iwl.hour_from IS NOT NULL AND iwl.duration IS NOT NULL
                    AND iwl.id NOT IN %s AND td.teacher_id IN %s
                ORDER BY td.teacher_id
                """
                cr.execute(query, (tuple(iwl_ids),tuple(teacher_ids),) )
                teacher_data={}
                for (teacher_id,week_day,hour_from,hour_to) in cr.fetchall():
                    if teacher_id not in teacher_data: teacher_data[room_id]=set()
                    week_day_name=None
                    for (day_code,day_name) in [('1','monday'),('2','tuesday'),('3','wednesday'),('4','thursday'),('5','friday'),('6','saturday'),('7','sunday')]:
                        if day_code==week_day: week_day_name=day_name
                    teacher_data[teacher_id].add( (week_day_name,hour_from,hour_to) )
                for item in self.pool.get('school.teacher').browse(cr, uid, teacher_data.keys(), context=context):
                    ctnat=etree.SubElement(tc_list,"ConstraintTeacherNotAvailableTimes")
                    subelement=etree.SubElement(ctnat,"Weight_Percentage")
                    subelement.text="100"
                    teacher_key="%s,%s" % (item.id,item.name)
                    subelement=etree.SubElement(ctnat,"Teacher")
                    subelement.text=teacher_key
                    subelement=etree.SubElement(ctnat,"Number_of_Not_Available_Times")
                    subelement.text=str(len(teacher_data[item.id]))
                    for (week_day,hour_from,hour_to) in teacher_data[item.id]:
                        subelement=etree.SubElement(ctnat,"Not_Available_Time")
                        subelement2=etree.SubElement(subelement,"Day")
                        subelement2.text=week_day
                        subelement2=etree.SubElement(subelement,"Hour")
                        subelement2.text=floats_to_school_fet_hour(hour_from,hour_to)

            # All activities with rooms
            sc_list=etree.SubElement(root2,"Space_Constraints_List")
            cbcs=etree.SubElement(sc_list,"ConstraintBasicCompulsorySpace")
            wp=etree.SubElement(cbcs,"Weight_Percentage")
            wp.text="100"
            if len(rooms)>1:
                for group_key in groups.keys():
                    csshr=etree.SubElement(sc_list,"ConstraintStudentsSetHomeRooms")
                    subelement=etree.SubElement(csshr,"Weight_Percentage")
                    subelement.text="10"
                    subelement=etree.SubElement(csshr,"Students")
                    subelement.text=group_key
                    subelement=etree.SubElement(csshr,"Number_of_Preferred_Rooms")
                    subelement.text=str(len(rooms))
                    for room_key in rooms:
                        subelement=etree.SubElement(csshr,"Preferred_Room")
                        subelement.text=room_key

            # Not available room hours
            query="""
            SELECT iwl.room_id,iwl.week_day,sfh.hour_from,sfh.hour_to
            FROM school_impartition_week_line AS iwl
                INNER JOIN school_fet_hour AS sfh ON iwl.hour_from<sfh.hour_to AND iwl.hour_from+iwl.duration>sfh.hour_from
            WHERE iwl.room_id IS NOT NULL AND iwl.week_day IS NOT NULL AND iwl.hour_from IS NOT NULL AND iwl.duration IS NOT NULL
                AND iwl.id NOT IN %s
            ORDER BY iwl.room_id
            """
            cr.execute(query, (tuple(iwl_ids),) )
            room_data={}
            for (room_id,week_day,hour_from,hour_to) in cr.fetchall():
                if room_id not in room_data: room_data[room_id]=set()
                week_day_name=None
                for (day_code,day_name) in [('1','monday'),('2','tuesday'),('3','wednesday'),('4','thursday'),('5','friday'),('6','saturday'),('7','sunday')]:
                    if day_code==week_day: week_day_name=day_name
                room_data[room_id].add( (week_day_name,hour_from,hour_to) )
            for item in self.pool.get('school.room').browse(cr, uid, room_data.keys(), context=context):
                crnat=etree.SubElement(sc_list,"ConstraintRoomNotAvailableTimes")
                subelement=etree.SubElement(crnat,"Weight_Percentage")
                subelement.text="100"
                room_key="%s,%s" % (item.id,item.name)
                subelement=etree.SubElement(crnat,"Room")
                subelement.text=room_key
                subelement=etree.SubElement(crnat,"Number_of_Not_Available_Times")
                subelement.text=str(len(room_data[item.id]))
                for (week_day,hour_from,hour_to) in room_data[item.id]:
                    subelement=etree.SubElement(crnat,"Not_Available_Time")
                    subelement2=etree.SubElement(subelement,"Day")
                    subelement2.text=week_day
                    subelement2=etree.SubElement(subelement,"Hour")
                    subelement2.text=floats_to_school_fet_hour(hour_from,hour_to)

                    
            
            self.write(cr, uid, [fex.id], {'file': base64.encodestring(etree.tostring(root2)) })
    
    def import_file(self, cr, uid, ids, context=None):
        for fex in self.browse(cr, uid, ids, context=context):            
            root=etree.fromstring(base64.decodestring(fex.file))
            iwl_data={}
            for data in root.findall('Time_Constraints_List/ConstraintActivityPreferredStartingTime'):
                iwl_id=int(data.find('Activity_Id').text)
                dia=data.find('Preferred_Day').text
                hora=data.find('Preferred_Hour').text
                week_day=False
                for (week_day_candidate,week_day_name) in [('1','monday'),('2','tuesday'),('3','wednesday'),('4','thursday'),('5','friday'),('6','saturday'),('7','sunday')]:
                    if week_day_name==dia: week_day=week_day_candidate

                hours=hora.split('-')
                if week_day and len(hours)>1:
                    (hour,minute)=hours[0].split(':')
                    hour_ids=self.pool.get('school.fet.hour').search(cr, uid, [('hour_from','=',str(float(hour)+float(minute)/60))])
                    for hour_data in self.pool.get('school.fet.hour').browse(cr, uid, hour_ids):
                        iwl_data[iwl_id]={'week_day': week_day, 'hour_from': hour_data.hour_from, }

            for data in root.findall('Space_Constraints_List/ConstraintActivityPreferredRoom'):
                iwl_id=int(data.find('Activity_Id').text)
                room=data.find('Room').text
                room_parts=room.split(',',1)
                if len(room_parts)==2:
                    iwl_data[iwl_id]['room_id']=int(room_parts[0])
                    
            for iwl_id in iwl_data.keys():
                self.pool.get('school.impartition_week_line').write(cr, uid, [iwl_id], iwl_data[iwl_id])

school_fet_export()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
