# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields

_MIN_DATETIME='1970-01-01 00:00:00'
_MAX_DATETIME='2099-12-31 23:59:59'

def _inclou_interval_en_llista(llista, interval):
    ret=[]
    (dtf0,dtt0)=interval
    afegit=False
    for (dtf,dtt) in llista:
        if dtt<dtf0:
            ret.append( (dtf,dtt) )
        elif dtt0<dtf:
            if not afegit:
                afegit=True
                ret.append( (dtf0, dtt0 ) )
            ret.append( (dtf,dtt) )
        else:
            dtf0=min(dtf,dtf0)
            dtt0=max(dtt,dtt0)
    if not afegit: ret.append( (dtf0, dtt0 ) )
    return ret

def _complementa_llista(llista_intervals_ordenada,interval):
    (dtf0,dtt0)=interval
    ret=[(dtf0,dtt0)]
    for (dtf,dtt) in llista_intervals_ordenada:
        if dtt>dtf0 and dtf<dtt0:
            if dtf>dtf0:
                ret[-1][1]=dtf
                ret.append((dtt,dtt0))
    return ret
        
        
class groups_group(osv.osv):
    _name = 'groups.group'

groups_group()

class groups_participation(osv.osv):
    _name = 'groups.participation'

groups_participation()

class groups_group_assignation(osv.osv):
    _name = 'groups.group_assignation'

    _columns =  {
        'group_id' : fields.many2one('groups.group','Group', required=True,
            select=1, ondelete='cascade',),
        'participation_id' : fields.many2one('groups.participation', 'Participation',
            required=True, select=1, ondelete='cascade'),
        'datetime_from' : fields.datetime('Begining', select=1,),
        'datetime_to' : fields.datetime('End', select=1,),
    }

    _sql_constraints = [('dates_ok','CHECK (datetime_to>=datetime_from)','Date to is minor than date from'),]

    def _assignation_in_domain(self, cr, uid, ids):
        data=self.read(cr, uid, ids, ['group_id','participation_id','datetime_from','datetime_to'], context={'withoutBlank': False})
        gids=set()
        for item in data:
            if item['datetime_from']!=item['datetime_to']: gids.add(item['group_id'])
        domains=self.pool.get('groups.group')._domain_assignations(cr, uid, list(gids))
        if domains:
            for item in data:
                group_id=item['group_id']
                participation_id=item['participation_id']
                if (group_id not in domains) or (domains[group_id] is None): continue
                # No participation in domain group
                if not participation_id in domains[group_id]: return False
                interval_into=(item['datetime_from'],item['datetime_to'])
                ret=False
                for interval_domain in domains[group_id][participation_id]:
                    if interval_into[0]>=interval_domain[0] and interval_into[1]<=interval_domain[1]: ret=True
                # No inclusive intervals in domain group for participation_id
                if not ret: return False
        return True

    _constraints = [
        (_assignation_in_domain, 'The assignation is outside the domain to the parent brand.', ['datetime_from','datetime_to']),
        ]

groups_group_assignation()

class groups_participation(osv.osv):
    _name = 'groups.participation'

    _columns = {
        'name' : fields.char('Ref',size=32),
        'participant' : fields.many2one('res.partner.contact', 'Participant',
            required=True, ondelete="cascade", select=1,
            help="The contact of the participation"),
        'assignation_ids' : fields.one2many('groups.group_assignation',
            'participation_id','Assignations',),
    }

    _defaults = {
        'name': lambda obj, cr, uid, context:
            obj.pool.get('ir.sequence').get(cr, uid, 'groups.ref.participation'),
    }

groups_participation()

class groups_classification(osv.osv):
    _name = 'groups.classification'
    
    _columns = {
        'name' : fields.char('Name',size=50,),
        'groups_rel_ids' : fields.one2many('groups.groups_rel','classification',string='Groups rel',),
    }
groups_classification()

class groups_groups_rel(osv.osv):
    _name = 'groups.groups_rel'
    
    _columns = {
        'parent_id' : fields.many2one('groups.group',string='Parent',ondelete='cascade',),
        'child_id' : fields.many2one('groups.group',string='Child',ondelete='cascade',),
        'classification' : fields.many2one('groups.classification',string='Classification'),
        'direction' : fields.selection([('up','Up'),('down','Down')],string='Direction',),
    }

    defaults = {
        'direction' : lambda *a: 'down',
    }

    # Each classification divides only one group.
    def _check_classification(self, cr, uid, ids):
        cr.execute("""select classification from (
                        select distinct classification,parent_id from groups_groups_rel where classification in (
                            select distinct classification from groups_groups_rel where id in %s )
                        ) as table1 group by classification having count(*)>1""", (tuple(ids),) )
        if cr.fetchall(): return false
        return True

    # You can not create recursive groups.
    def _check_recursion(self, cr, uid, ids):
        level = 100
        while len(ids):
            cr.execute('select distinct parent_id from groups_groups_rel where child_id in %s', (tuple(ids),))
            ids = filter(None, map(lambda x:x[0], cr.fetchall()))
            if not level:
                return False
            level -= 1
        return True


    _constraints = [
        (_check_recursion, 'Error ! You can not create recursive groups.', ['parent_ids']),
        (_check_classification, 'Each classification divides only one group.',['classification'])
    ]

groups_groups_rel()

class groups_group(osv.osv):
    _name = 'groups.group'
        
    _columns = {
        'name' : fields.char('Name', size=100, required=True, select=1, help="The group name"),
        'parent_ids' : fields.one2many('groups.groups_rel','child_id',string='Parent relations',help="The parent relations"),
        'children_ids' : fields.one2many('groups.groups_rel','parent_id',string='Children relations',help="The children relations"),
        'assignation_ids' : fields.one2many('groups.group_assignation','group_id','Assignations'),
        'priority' : fields.integer('Priority',required=True,),
        'creation' : fields.integer('Auto creation', required=True,),
    }

    _defaults = {
        'creation': lambda *a: 0,
        'priority': lambda *a: 0,
    }

    def _domain_assignations(self, cr, uid, ids, context=None):

        ret={}
        domains={}
        for item in self.browse(cr, uid, ids, context={'withoutBlank': False}):
            id=item.id
            ret[id]={}
            if item.parent_ids:
                for parent_rel in item.parent_ids:
                    parent=parent_rel.parent_id
                    pid=parent.id
                    if not pid in domains: # per estalviar-nos cercar dominis de pares iguals
                        dict={}
                        for assignation in parent.assignation_ids:
                            key=assignation.participation_id.id
                            if not key in dict: dict[key]=[]
                            dict[key].append( (assignation.datetime_from,assignation.datetime_to) )
                        domains[pid]=dict
                    for (key,value) in domains[pid].items():
                        if key not in ret[id]: ret[id][key]=[]
                        for interval in value:
                            ret[id][key]=_inclou_interval_en_llista(ret[id][key], interval)
            else:
                ret[id]=None # No parent, no domain
        return ret


groups_group()
