import math
# -*- coding: utf-8 -*-
##############################################################################
#
#    school module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#    This file is a part of school module
#
#    school OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    school OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
from timed_groups import _inclou_interval_en_llista,_complementa_llista,_MIN_DATETIME,_MAX_DATETIME

#class groups_group(osv.osv):
#    _name = 'groups.group'
#
#groups_group()
#
#class groups_participation(osv.osv):
#    _name = 'groups.participation'
#
#groups_participation()

_CHANGE_ASSIGNATION_GROUP_IDS_KEY='change_assignation_group_ids'
_CLASSIFYING_KEY='classifying'

class groups_group_assignation(osv.osv):
    _name = 'groups.group_assignation'
    _inherit = 'groups.group_assignation'

    def add_assignation(self, cr, uid, participation_id, group, datetime_from=_MIN_DATETIME, datetime_to=_MAX_DATETIME, context=None):
        if not context: context={}
        ret=None
        datetime_from=max(_MIN_DATETIME,datetime_from or _MIN_DATETIME)
        datetime_to=min(_MAX_DATETIME,datetime_to or _MAX_DATETIME)
        if type(group) in (int,long):
            group=self.pool.get('groups.group').browse(cr, uid, group, context={'withoutBlank': False })

        # Don't repeat group
        if not _CHANGE_ASSIGNATION_GROUP_IDS_KEY in context:
            context[_CHANGE_ASSIGNATION_GROUP_IDS_KEY]=[]
        if group.id in context[_CHANGE_ASSIGNATION_GROUP_IDS_KEY]:
            return
        context[_CHANGE_ASSIGNATION_GROUP_IDS_KEY].append(group.id)

        # Els pares que baixen domini cap el grup resulten amb un domini intersecció
        unio_dels_dominis_complementaris=[]
        for parent_rel in group.parent_ids:
            if parent_rel.direction=='down':
                ga_ids=self.pool.get('groups.group_assignation').search(cr, uid, [('group_id','=',parent_rel.parent_id.id),('participation_id','=',participation_id),('datetime_from','>',datetime_to),('datetime_to','<',datetime_from)])
                for ga in self.browse(cr, uid, ga_ids, context={'withoutBlank': False,}):
                    if ga.datetime_from>datetime_from:
                        _inclou_interval_en_llista(unio_dels_dominis_complementaris,(datetime_from,ga.datetime_from))
                    if ga.datetime_from<datetime_from:
                        _inclou_interval_en_llista(unio_dels_dominis_complementaris,(ga.datetime_to,datetime_to))

        # De la intersecció potser surten més d'un interval
        for (dt_from,dt_to) in _complementa_llista(unio_dels_dominis_complementaris,(datetime_from,datetime_to)):
            # Amplia assignacions dels grups superiors per la participació
            # Si hi ha un pare sense classificació ha de ser pare únic perquè en cas contrari no sabem a quin pare hem de amplia l'assignació
            for parent_rel in group.parent_ids:
                if parent_rel.direction=='up':
                    self.add_assignation(cr, uid, participation_id, parent_rel.parent_id.id, datetime_from=dt_from, datetime_to=dt_to, context=context)
                if parent_rel.classification:
                    for group_rel in parent_rel.classification.groups_rel_ids:
                        if group_rel.child_id.id!=group.id:
                            self.remove_assignation(cr, uid, participation_id, group_rel.child_id.id, datetime_from=dt_from, datetime_to=dt_to, context=context)


            # Fusiona assignacions dins de l'interval respectant els límits de les ja incloses
            ga_ids=self.search(cr, uid, [('group_id','=',group.id),('participation_id','=',participation_id),('datetime_from','<',dt_to),('datetime_to','>',dt_from)])
            if 'ga_id' in context and context['ga_id'] in ga_ids:
                ret=context['ga_id']
            datetime_from_min=dt_from
            datetime_to_max=dt_to
            for item in self.browse(cr, uid, ga_ids, context={'withoutBlank': False,}):
                datetime_from_min=min(item.datetime_from or _MIN_DATETIME,datetime_from_min)
                datetime_to_max=max(item.datetime_to or _MAX_DATETIME,datetime_to_max)
            vals={'participation_id': participation_id, 'group_id': group.id, 'datetime_from': datetime_from_min, 'datetime_to': datetime_to_max,}
            if ret:
                super(groups_group_assignation, self).write(cr, uid, [ret], vals)
                ga_ids.remove(ret)
            else:
                ret=super(groups_group_assignation, self).create(cr, uid, vals)
            super(groups_group_assignation, self).unlink(cr, uid, ga_ids)

            # Decidim a quin grup de les divisions afegim l'assignació
            for classification in set(child_rel.classification for child_rel in group.children_ids if child_rel.direction=='down' and child_rel.classification):
                self.pool.get('groups.classification').add_assignation(cr, uid, participation_id, classification,datetime_from=dt_from, datetime_to=dt_to, context=context)
        return ret


    def remove_assignation(self, cr, uid, participation_id, group, datetime_from=_MIN_DATETIME, datetime_to=_MAX_DATETIME, context=None):
        if not context: context={}
        datetime_from=max(_MIN_DATETIME,datetime_from or _MIN_DATETIME)
        datetime_to=min(_MAX_DATETIME,datetime_to or _MAX_DATETIME)
        
        if type(group) in (int,long):
            group=self.pool.get('groups.group').browse(cr, uid, group, context={'withoutBlank': False,})

        if not _CHANGE_ASSIGNATION_GROUP_IDS_KEY in context:
            context[_CHANGE_ASSIGNATION_GROUP_IDS_KEY]=[]
        if group.id in context[_CHANGE_ASSIGNATION_GROUP_IDS_KEY]:
            return
        context[_CHANGE_ASSIGNATION_GROUP_IDS_KEY].append(group.id)
        
        # Treu l'assignació a tots els grups inferiors
        for child_rel in group.children_ids:
            if child_rel.classification and child_rel.direction=='up':
                raise orm.except_orm('Problem',"I couldn,t remove assignation from a agrupation.")
            self.remove_assignation(cr, uid, participation_id, child_rel.child_id, datetime_from=datetime_from, datetime_to=datetime_to, context=context)

        ga_ids=self.search(cr, uid, [('group_id','=',group.id),('participation_id','=',participation_id),('datetime_from','<',datetime_to),('datetime_to','>',datetime_from)])
        datetime_from_min=datetime_from
        datetime_to_max=datetime_to
        for data in self.read(cr, uid, ga_ids, ['datetime_from','datetime_to'], context={'withoutBlank': False,}):
            datetime_from_min=min(data['datetime_from'],datetime_from_min)
            datetime_to_max=max(data['datetime_to'],datetime_to_max)
        if datetime_from_min<datetime_from:
            super(groups_group_assignation, self).create(cr, uid, {'participation_id': participation_id, 'group_id': group.id, 'datetime_from': datetime_from_min, 'datetime_to': datetime_from,})
        if datetime_to_max<datetime_to:
            super(groups_group_assignation, self).create(cr, uid, {'participation_id': participation_id, 'group_id': group.id, 'datetime_from': datetime_to, 'datetime_to': datetime_to_max,})
        if 'ga_id' in context and context['ga_id'] in ga_ids:
            ga_ids.remove(context['ga_id'])
        super(groups_group_assignation, self).unlink(cr, uid, ga_ids)
        
        # Treiem l'assignació de tots els pares del qual aquest és un grup de la classificació
        for parent_rel in group.parent_ids:
            if parent_rel.classification and not context.get(_CLASSIFYING_KEY,False):
                if parent_rel.direction=='down':
                    raise orm.except_orm('Problem',"I couldn,t remove assignation from a classification.")
                self.remove_assignation(cr, uid, participation_id, parent_rel.parent_id, datetime_from=datetime_from, datetime_to=datetime_to, context=context)

    def read(self, cr, uid, ids, fields, context=None, load='_classic_write'):
        if not fields: fields=[]
        if not context: context={}
        ret=super(groups_group_assignation, self).read(cr, uid, ids, fields, context=context, load=load)
        # @type context dict
        if context.get('withoutBlank', True):
            # @type ret list
            for item in ret:
                if 'datetime_from' in item:
                    if item['datetime_from']==_MIN_DATETIME: item['datetime_from']=''
                if 'datetime_to' in item:
                    if item['datetime_to']==_MAX_DATETIME: item['datetime_to']=''
        return ret
        
    def create(self, cr, uid, vals, context=None):
        return self.add_assignation(cr, uid, vals['participation_id'], vals['group_id'],
            datetime_from=vals.get('datetime_from',_MIN_DATETIME),
            datetime_to=vals.get('datetime_to',_MAX_DATETIME), context=context)

    def write(self, cr, uid, ids, vals, context=None):
        if not context: context={}
        for item in self.browse(cr, uid, ids, {'withoutBlank': False,}):
            context.update({'ga_id': item.id,})
            self.remove_assignation(cr, uid, item.participation_id, item.group_id, datetime_from=item.datetime_from, datetime_to=item.datetime_to, context=context)
        super(groups_group_assignation, self).write(cr, uid, ids, vals, context=context)
        for item in self.browse(cr, uid, ids, {'withoutBlank': False,}):
            context.update({'ga_id': item.id,})
            self.add_assignation(cr, uid, item.participation_id, item.group_id, datetime_from=item.datetime_from, datetime_to=item.datetime_to, context=context)

    def unlink(self, cr, uid, ids, context=None):
        context[_CHANGE_ASSIGNATION_GROUP_IDS_KEY]=[]
        for item in self.browse(cr, uid, ids, {'withoutBlank': False,}):
            self.remove_assignation(cr, uid, item.participation_id.id, item.group_id, datetime_from=item.datetime_from, datetime_to=item.datetime_to, context=context)
        return True

groups_group_assignation()

class groups_participation(osv.osv):
    _name = 'groups.participation'
    _inherit = 'groups.participation'


    def _get_groups(self, cr, uid, ids, field_name, arg, context=None):
        ret={}
        for obj in self.browse(cr, uid, ids):
            reti=set()
            retxt=[]
            for obj2 in sorted(obj.assignation_ids, key=lambda assig: assig.group_id.name):
                reti.add(obj2.group_id.id)
                retxt.append(obj2.group_id.name)
            ret[obj.id]={'group_ids': list(reti), 'group_txt': ','.join(retxt)}
        return ret

    def _search_by_group(self, cr, uid, obj, name, args, context=None):
        ga_obj=self.pool.get('groups.group_assignation')
        ga_ids=ga_obj.search(cr, uid, [('group_id.name',op,value) for (namex,op,value) in args if namex==name])
        return [('id','in',[x['participation_id'] for x in ga_obj.read(cr, uid, ga_ids, ['participation_id'])])]

    _columns = {
        'group_ids' : fields.function(_get_groups, fnct_search=_search_by_group, arg=None, type='many2many', obj='groups.group', method=True, string="Groups", multi='groups',),
        'group_txt' : fields.function(_get_groups, arg=None, type='char', method=True, string="Groups with assignation", multi='groups',),
    }

    def groups_in_interval(self, cr, uid, ids, date_from=_MIN_DATETIME, date_to=_MAX_DATETIME, context=None):
        """
        Retorna un diccionari amb claus els identificadors de les participacions
        contingudes en la llista del paràmetre ids. Els valors són una llista de
        diccionaris amb dades sobre els grups amb assignacions per la participació
        identificada amb la clau dins el periode determinat pels paràmetres
        date_from i date_to. La llista de grups es troba ordenada per prioritat
        éssent el grup amb prioritat més alta el quin és primer.
        """
        if not ids: return {}
        res={}
        for id in ids: res[id]=[]
        if ids:
            query="""
            SELECT ga.participation_id,ga.datetime_from,ga.datetime_to,g.id AS gid,g.priority
            FROM groups_group_assignation AS ga
            INNER JOIN groups_group AS g ON ga.group_id=g.id
            WHERE ga.participation_id in %(part_ids)s
                AND ga.limit_from<'%(date_to)s'
                AND ga.limit_to>'%(date_from)s'
            ORDER BY g.priority,ga.datetime_from DESC
            """ % {'part_ids': tuple(ids+[0]),'date_from': date_from, 'date_to': date_to, }
            cr.execute(query)
            for (part_id,datetime_from,datetime_to,group_id,priority) in cr.fetchall():
                if part_id not in res: res[part_id]=[]
                res[part_id]+=[{'group_id': group_id,'datetime_from': datetime_from,'datetime_to': datetime_to,'priority':priority}]
        return res

groups_participation()

selection_methods=[('manual','Manual'),('alfa_classi','Alphabetical classification'),]

class groups_classification(osv.osv):
    _name = 'groups.classification'
    _inherit = 'groups.classification'
    
    _columns = {
        'method' : fields.selection(selection_methods,string='Classification Method',),
    }

    def alfabetical_classification_method(self, cr, uid, classification, context=None):
        if not context: context={}
        # Escollim el grup que li toca dividint segons ordre alfabetic
        participations={}
        groups={}
        for groups_rel in classification.groups_rel_ids:
            group_domain=groups_rel.direction=='up' and groups_rel.child_id or groups_rel.parent_id
            group_child=groups_rel.direction=='down' and groups_rel.child_id or groups_rel.parent_id
            groups[(group_child.id,group_child.name)]=group_child
            for ga in group_domain.assignation_ids:
                key=(ga.participation_id.id,ga.participation_id.participant.name)
                if key not in participations:
                    participations[key]=[]
                participations[key].append( ((ga.datetime_from,ga.datetime_to),ga.group_id) )
        participations_sorted=sorted(participations.keys(),key=lambda x: x[1])
        groups_sorted=sorted(groups.keys(),key=lambda x: x[1])
        c=0
        for (part_id,contact_name) in participations_sorted:
            grup_ind=int(c*len(groups.keys())/len(participations_sorted))
            if participations[(part_id,contact_name)]:
                for (dates,group) in participations[(part_id,contact_name)]:
                    if groups_sorted[grup_ind][0]!=group.id:
                        context[_CHANGE_ASSIGNATION_GROUP_IDS_KEY]=[]
                        context[_CLASSIFYING_KEY]=True
                        self.pool.get('groups.group_assignation').add_assignation(cr, uid, part_id, groups[groups_sorted[grup_ind]], datetime_from=dates[0], datetime_to=dates[1], context=context)
                        context[_CLASSIFYING_KEY]=False
            else:
                context[_CHANGE_ASSIGNATION_GROUP_IDS_KEY]=[]
                context[_CLASSIFYING_KEY]=True
                self.pool.get('groups.group_assignation').add_assignation(cr, uid, part_id, groups[groups_sorted[grup_ind]], datetime_from=datetime_from, datetime_to=datetime_to, context=context)
                context[_CLASSIFYING_KEY]=False
            c+=1


    def add_assignation(self, cr, uid, participation_id, classification, datetime_from=_MIN_DATETIME, datetime_to=_MAX_DATETIME, context=None):
        for groups_rel in classification.groups_rel_ids:
            target_group=groups_rel.direction=='up' and groups_rel.parent_id or groups_rel.child_id
            self.pool.get('groups.group_assignation').add_assignation(cr, uid, participation_id, target_group, datetime_from=_MIN_DATETIME, datetime_to=_MAX_DATETIME, context=context)
            break
        self.classify(cr, uid, classification, context=context)

    def classify(self, cr, uid, classification, context=None):
        if classification.method=='alfa_classi':
            self.alfabetical_classification_method(cr, uid, classification, context=context)

groups_classification()

class groups_groups_rel(osv.osv):
    _name = 'groups.groups_rel'
    _inherit = 'groups.groups_rel'
    
    def create(self, cr, uid, vals, context=None):
        if not context: context={}
        ret=super(groups_groups_rel, self).create(cr, uid, vals, context=context)
        item=self.browse(cr, uid, ret, context={'withoutBlank': False,})
        if item.classification:
            self.pool.get('groups.classification').classify(cr, uid, item.classification, context=context)
        if item.direction=='up':
            for ga in item.child_id.assignation_ids:
                context[_CHANGE_ASSIGNATION_GROUP_IDS_KEY]=[item.child_id.id]
                self.pool.get('groups.group_assignation').add_assignation(cr, uid, ga.participation_id.id, item.parent_id, datetime_from=ga.datetime_from, datetime_to=ga.datetime_to, context=context)
        if not self.pool.get('groups.group_assignation')._assignation_in_domain(cr, uid, [ga.id for ga in item.child_id.assignation_ids]):
            raise orm.except_orm('Problem',"El domini del pare no inclou el domini del fill.")

    def write(self, cr, uid, ids, vals, context=None):
        raise orm.except_orm('Problem',"You could only create or remove group relationship.")

    def unlink(self, cr, uid, ids, context=None):
        for item in self.browse(cr, uid, ids, context={'withoutBlank': False,}):
            source_group=item.direction=='down' and item.parent_id or item.child_id
            target_group=item.direction=='up' and item.parent_id or item.child_id
            for ga in source_group.assignation_ids:
                context[_CHANGE_ASSIGNATION_GROUP_IDS_KEY]=[]
                self.pool.get('groups.group_assignation').remove_assignation(cr, uid, ga.participation_id.id, target_group, ga.datetime_from, ga.datetime_to, context=context)
            
groups_groups_rel()

class groups_group(osv.osv):
    _name = 'groups.group'
    _inherit = 'groups.group'

    def _participants(self, cr, uid, ids, field_name, arg, context=None):
        ret={}
        for id in ids: ret[id]=[]
        query='select group_id,participation_id from groups_group_assignation where group_id in %s'
        cr.execute(query, (tuple(ids,),))
        for (gid,pid) in cr.fetchall(): ret[gid].append(pid)
        return ret

    def _compute_name2(self, cr, uid, ids, field_name, arg, context=None):
        separator_levels='/'
        separator_groups=','
        separator_classification='+'
        ret=dict.fromkeys(ids,'')
        query="SELECT g.id,g.name,gr.parent_id,gr.classification FROM groups_group AS g LEFT JOIN groups_groups_rel AS gr ON gr.child_id=g.id WHERE g.id IN %s"
        cr.execute(query, (tuple(ids),))
        parent_ids=set()
        parents={}
        for (id,name,parent_id,classification) in cr.fetchall():
            ret[id]=name
            if parent_id: parent_ids.add(parent_id)
            if id not in parents: parents[id]={}
            if classification not in parents[id]: parents[id][classification or 0]=[]
            if parent_id: parents[id][classification or 0].append(parent_id)
        if parent_ids:
            parent_names=self._compute_name2(cr, uid, list(parent_ids), field_name, arg, context)
            for id in ids:
                if id in parent_ids:
                    name=''
                    for (classification,parent_ids2) in sorted(parents[id].items(),key=lambda x: x[0]):
                        classification_parent_names=map(lambda x: parent_names[x],parent_ids2)
                        sep=separator_groups or classification!=0 and separator_classification
                        name+=sep.join(classification_parent_names)
                    ret[id]=name+separator_levels+ret[id]
        return ret
            
    def _get_group_ids_and_our_children(self, cr, uid, ids, context=None):
        if not context: context={}
        context.setdefault('_get_group_ids_and_our_children_proof', 0)
        if not ids: return []
        if context['_get_group_ids_and_our_children_proof']>100:
            raise orm.except_orm('Problem',"Max indeep recursion.")
        context['_get_group_ids_and_our_children_proof']+=1
        ret=set(ids)
        query="SELECT child_id FROM groups_groups_rel WHERE parent_id IN %s"
        cr.execute(query, (tuple(ret),))
        ret.update(self._get_group_ids_and_our_children(cr, uid, [x for (x,) in cr.fetchall()], context=context))
        return list(ret)
        
    _columns = {
        'name2' : fields.function(_compute_name2, type='char', size=200, method=True, string='Complete name', select=1, store={'groups.group': (_get_group_ids_and_our_children, ['name'], 10 ),}),
        'participants' : fields.function(_participants, method=True, type='one2many',relation='groups.participation'),
    }


groups_group()
